#include "testlib.h"
#include <bits/stdc++.h>

using namespace std;

map<string, string> params;

long long get(string name, long long def = 0) {
    if (params.count(name)) {
        return stoll(params[name]);
    } else {
        return def;
    }
}

char get_ch(string name, char def = 'a') {
    if (params.count(name)) {
        return params[name][0];
    } else {
        return def;
    }
}

const int MAXN = 2e5;
const int mod = 1e9 + 7;
int pr[MAXN];
vector<int> query[MAXN];
vector<vector<int>> g;
pair<int, int> seg[MAXN];

pair<int, int> get_pair() {
    pair<int, int> res = {rnd.next(1, mod - 1), rnd.next(1, mod - 1)};
    if (res.first > res.second) {
        swap(res.first, res.second);
    }
    return res;
}

void dfs(int v, int p) {
    pr[v] = p;
    for (auto u : g[v]) {
        if (u != p) {
            dfs(u, v);
        }
    }
}

class LCA_log {
public:
    int n, LG;
    vector<int> dist;
    vector<vector<int>> g, up;

    void dfs(int v, int pr, int d) {
        dist[v] = d;
        up[v].resize(LG);
        up[v][0] = pr;
        for (int j = 1; j < LG; j++) {
            up[v][j] = up[up[v][j - 1]][j - 1];
        }
        for (auto x : g[v]) {
            if (x != pr) {
                dfs(x, v, d + 1);
            }
        }
    }

    LCA_log(vector<vector<int>>& _g, int st) {
        n = (int) _g.size();
        g = _g;
        LG = 0;
        while ((1ll << LG) <= n) LG++;
        up.resize(n);
        dist.resize(n);
        dfs(st, st, 0);
    }

    int la(int x, int z) {
        for (int j = 0; j < LG; j++) {
            if ((z & (1 << j)) > 0) {
                x = up[x][j];
            }
        }
        return x;
    }

    int lca(int x, int y) {
        if (dist[x] < dist[y]) swap(x, y);
        int z = dist[x] - dist[y];
        for (int j = 0; j < LG; j++) {
            if ((z & (1 << j)) > 0) {
                x = up[x][j];
            }
        }
        if (x == y) return x;
        for (int j = LG - 1; j >= 0; j--) {
            int X = up[x][j], Y = up[y][j];
            if (X != Y) {
                x = X, y = Y;
            }
        }
        return up[x][0];
    }
    int distance(int x, int y) {
        return dist[x] + dist[y] - 2 * dist[lca(x, y)];
    }
};

class LCA {
public:
    int n;
    vector<pair<int, int>> st;
    vector<int> tin, dist, lg;
    vector<vector<int>> g;
    vector<vector<pair<int, int>>> ST;

    void dfs(int v, int pr, int d) {
        tin[v] = st.size();
        dist[v] = d;
        st.push_back({d, v});
        for (auto x : g[v]) {
            if (x == pr) continue;
            dfs(x, v, d + 1);
            st.push_back({d, v});
        }
    }

    LCA(vector<vector<int>>& _g, int stv) {
        n = (int) _g.size();
        g = _g;
        tin.resize(n);
        dist.resize(n);
        dfs(stv, -1, 0);
        int m = st.size();
        lg.resize(m + 1);
        for (int i = 2; i <= m; i++) lg[i] = lg[i / 2] + 1;
        int maxlg = lg[m] + 1;
        ST.resize(maxlg);
        ST[0] = st;
        for (int i = 1; i < maxlg; i++) {
            ST[i].resize(m - (1 << i) + 1);
            for (int j = 0; j < m - (1 << i) + 1; j++) {
                ST[i][j] = min(ST[i - 1][j], ST[i - 1][j + (1 << (i - 1))]);
            }
        }
    }

    int lca(int x, int y) {
        int l = min(tin[x], tin[y]), r = max(tin[x], tin[y]);
        int LG = lg[r - l + 1];
        pair<int, int> res = min(ST[LG][l], ST[LG][r - (1 << LG) + 1]);
        return res.second;
    }

    int distance(int x, int y) {
        return dist[x] + dist[y] - 2 * dist[lca(x, y)];
    }
};

int main(int argc, char* argv[]) {
    registerGen(argc, argv, 1);
    for (int i = 1; i < argc; i++) {
        string s = string(argv[i]);
        if (s.find("=") != string::npos) {
            auto pos = s.find("=");
            params[s.substr(0, pos)] = s.substr(pos + 1);
        } else {
            params[s] = "";
        }
    }

    string treemode = params["treemode"];
    string nummode = params["nummode"];
    string pathmode = params["pathmode"];
    int n = get("N", MAXN);
    int q = get("Q", MAXN);

    g.resize(n);

    if (nummode == "same") {
        auto [l, r] = get_pair();
        for (int i = 0; i < n; i++) {
            seg[i] = {l, r};
        }
    } else if (nummode == "rand") {
        for (int i = 0; i < n; i++) {
            seg[i] = get_pair();
        }
    } else if (nummode == "nice") {
        auto [l, r] = get_pair();
        for (int i = 0; i < n; i++) {
            seg[i] = {rnd.next(1, l), rnd.next(r, mod - 1)};
        }
    }  else if (nummode == "best") {
        auto [l, r] = get_pair();
        for (int i = 0; i < n; i++) {
            if (rnd.next(1, 100) == 1) {
                int l2 = rnd.next(l, r);
                int r2 = rnd.next(l, r);
                if (l2 > r2) {
                    swap(l2, r2);
                }
                if (r2 - l2 + 1 >= 2) {
                    l = l2, r = r2;
                }
            }
            seg[i] = {rnd.next(1, l), rnd.next(r, mod - 1)};
        }
    } else {
        assert(false);
    }

    if (treemode == "rand") {
        int wnext = get("twnext", 0);
        int maxdiff = get("maxdiff", n);
        for (int i = 1; i < n; i++) {
            int p = rnd.wnext(max(0, i - maxdiff), i - 1, wnext);
            g[i].push_back(p);
            g[p].push_back(i);
        }
    } else if (treemode == "bamboo") {
        for (int i = 1; i < n; i++) {
            g[i].push_back(i - 1);
            g[i - 1].push_back(i);
        }
    } else if (treemode == "prufer") {
        if (n >= 2) {
            vector<int> seq(n - 2);
            set<int> nxt;
            map<int, int> cnt;
            for (int i = 0; i < n - 2; i++) {
                seq[i] = rnd.next(n);
                cnt[seq[i]]++;
            }
            for (int i = 0; i < n; i++) {
                if (!cnt.count(i)) {
                    nxt.insert(i);
                }
            }
            for (int i = 0; i < n - 2; i++) {
                int a = *nxt.begin();
                nxt.erase(a);
                g[a].push_back(seq[i]);
                g[seq[i]].push_back(a);
                cnt[seq[i]]--;
                if (cnt[seq[i]] == 0) {
                    nxt.insert(seq[i]);
                }
            }
            int a = *nxt.begin();
            nxt.erase(a);
            int b = *nxt.begin();
            g[a].push_back(b);
            g[b].push_back(a);
        }
    } else if (treemode == "brush") {
        int x = 1;
        for (; x < n / 2; x++) {
            int p = max(0, x - 2);
            g[p].push_back(x);
            g[x].push_back(p);
        }
        for (int i = x; i < n; i++) {
            int p = x - 1;
            if (i % 2 == 0) {
                p = max(p - 1, 0);
            }
            g[p].push_back(i);
            g[i].push_back(p);
        }
    } else if (treemode == "hld") {
        function<void(int, int, int)> build = [&] (int root, int l, int r) {
            if (l >= r) {
                return;
            }
            int tmp = l;
            if (root != -1) {
                g[root].push_back(tmp);
                g[tmp].push_back(root);
            }
            int cnt = r - l;
            while (tmp < l + cnt / 4) {
                g[tmp + 1].push_back(tmp);
                g[tmp].push_back(tmp + 1);
                tmp++;
            }
            int vl = tmp + 1, vr = r;
            int vm = (vl + vr + 1) / 2;
            build(tmp, vl, vm);
            build(tmp, vm, vr);
        };
        build(-1, 0, n);
    } else if (treemode == "binary") {
        for (int i = 1; i < n; i++) {
            int x = i, y = (i - 1) / 2;
            g[y].push_back(x);
            g[x].push_back(y);
        }
    } else if (treemode == "smart_bamboo") {
        int cnt = n * ((long double) 4 / (long double) 5);
        while (cnt > n) {
            cnt--;
        }
        while (cnt < 1) {
            cnt++;
        }
        vector<int> p(n);
        iota(p.begin(), p.end(), 0);
        mt19937 gen(101);
        shuffle(p.begin(), p.end(), gen);
        for (int i = 0; i + 1 < cnt; i++) {
            int x = p[i], y = p[i + 1];
            g[y].push_back(x);
            g[x].push_back(y);
        }
        for (int i = cnt; i < n; i++) {
            int j = gen() % i;
            int x = p[i], y = p[j];
            g[y].push_back(x);
            g[x].push_back(y);
        }
        for (int i = 0; i < q; i++) {
            int len = rnd.next(min(i + 1, cnt), cnt);
            int v = rnd.next(0, cnt - len);
            int u = rnd.next(0, cnt - len);
            if (rnd.next(0, 1) == 0) {
                query[i].push_back(p[v]);
                query[i].push_back(p[v + len - 1]);
            } else {
                query[i].push_back(p[v + len - 1]);
                query[i].push_back(p[v]);
            }
            if (rnd.next(0, 1) == 0) {
                query[i].push_back(p[u]);
                query[i].push_back(p[u + len - 1]);
            } else {
                query[i].push_back(p[u + len - 1]);
                query[i].push_back(p[u]);
            }
        }
    } else {
        assert(false);
    }

    if (treemode != "bamboo" && treemode != "smart_bamboo") {
        auto g2 = g;
        g.clear();
        g.resize(n);
        vector<int> p(n);
        iota(p.begin(), p.end(), 0);
        mt19937 gen(101);
        shuffle(p.begin(), p.end(), gen);
        for (int i = 0; i < n; i++) {
            for (auto j : g2[i]) {
                g[p[i]].push_back(p[j]);
            }
        }
    }

    if (pathmode == "stupid") {
        LCA lca(g, 0);
        vector<pair<int, int>> z[n + 1];
        for (int i = 0; i < (int) 5e6; i++) {
            int x = rnd.next(0, n - 1);
            int y = rnd.next(0, n - 1);
            z[lca.distance(x, y)].push_back({x, y});
            z[lca.distance(x, y)].push_back({y, x});
        }
        vector<array<int, 4>> kek;
        for (int i = n; i >= 0; i--) {
            for (int x = 0; x < (int) z[i].size() && (int) kek.size() < q; x++) {
                for (int y = 0; y < (int) z[i].size() && (int) kek.size() < q; y++) {
                    kek.push_back({z[i][x].first, z[i][x].second, z[i][y].first, z[i][y].second});
                }
            }
        }
        if (kek.empty()) {
            kek.push_back({0, 0, 0, 0});
        }
        for (int i = 0; i < q; i++) {
            for (auto x : kek[i % (int) kek.size()]) {
                query[i].push_back(x);
            }
        }
    } else if (pathmode == "4group") {
        LCA lca(g, 0);
        vector<pair<int, int>> z[n + 1];
        for (int i = 0; i < (int) 5e6; i++) {
            int x = rnd.next(0, n - 1);
            int y = rnd.next(0, n - 1);
            z[lca.distance(x, y)].push_back({x, y});
        }
        vector<array<int, 4>> kek;
        for (int i = n; i >= 0; i--) {
            for (int x = 0; x < (int) z[i].size() && (int) kek.size() < q; x++) {
                for (int y = 0; y < (int) z[i].size() && (int) kek.size() < q; y++) {
                    kek.push_back({z[i][x].first, z[i][x].second, z[i][y].first, z[i][y].second});
                }
            }
        }
        if (kek.empty()) {
            kek.push_back({0, 0, 0, 0});
        }
        for (int i = 0; i < q; i++) {
            for (auto x : kek[i % (int) kek.size()]) {
                query[i].push_back(x);
            }
        }
        long long sum = 0;
        for (int i = 0; i < q; i++) {
            sum += 2 * lca.distance(query[i][0], query[i][1]);
        }
        long long limit = (long long) 1e8;
        for (int i = 0; i < q; i++) {
            if (sum > limit) {
                sum -= 2 * lca.distance(query[i][0], query[i][1]);
                query[i] = {0, 0, 0, 0};
                sum += 2;
            }
        }
    } else if (pathmode == "bamboo") {
        int dmin = get("dmin", 1);
        dmin = min(n, dmin);
        for (int i = 0; i < q; i++) {
            int len = rnd.next(dmin, n);
            int v = rnd.next(0, n - len);
            int u = rnd.next(0, n - len);
            if (rnd.next(0, 1) == 0) {
                query[i].push_back(v);
                query[i].push_back(v + len - 1);
            } else {
                query[i].push_back(v + len - 1);
                query[i].push_back(v);
            }
            if (rnd.next(0, 1) == 0) {
                query[i].push_back(u);
                query[i].push_back(u + len - 1);
            } else {
                query[i].push_back(u + len - 1);
                query[i].push_back(u);
            }
        }
    } else if (pathmode == "smart_bamboo") {
        while (q > n) {
            query[q - 1] = {0, 0, 0, 0};
            q--;
        }
        for (int i = 0; i < min(n, q); i++) {
            query[i].push_back(i);
            query[i].push_back(0);
            query[i].push_back(0);
            query[i].push_back(i);
        }
    } else if (pathmode == "binary") {
        vector<vector<int>> qu;
        vector<int> ts;
        function<void(int, int)> dfs = [&] (int v, int p) {
            if ((int) g[v].size() == 1) {
                ts.push_back(v);
            }
            for (auto u : g[v]) {
                if (u != p) {
                    dfs(u, v);
                }
            }
        };
        dfs(0, 0);
        int m = (int) ts.size();
        LCA lca(g, 0);
        LCA_log kek(g, 0);
        for (int j = 0; j < n; j++) {
            bool f = false;
            for (int i = 0; i < m / 2; i++) {
                int x = ts[i], y = ts[i + m / 2];
                if (lca.dist[x] < j || lca.dist[y] < j) {
                    f = true;
                    continue;
                }
                qu.push_back({x, kek.la(x, j), y, kek.la(y, j)});
            }
            if (f) {
                break;
            }
        }
        mt19937 gen(1337);
        shuffle(qu.begin(), qu.end(), gen);
        for (int i = 0; i < q; i++) {
            query[i] = qu[i % (int) qu.size()];
        }
    } else if (pathmode == "smart") {
        vector<int> ts;
        function<void(int, int)> dfs = [&] (int v, int p) {
            if ((int) g[v].size() == 1) {
                ts.push_back(v);
            }
            for (auto u : g[v]) {
                if (u != p) {
                    dfs(u, v);
                }
            }
        };
        dfs(0, 0);
        mt19937 gen(228);
        int m = (int) ts.size();
        LCA_log lca(g, 0);
        for (int i = 0; i < q; i++) {
            int x = ts[gen() % m];
            int y = ts[gen() % m];
            int d = min({i + 1, lca.dist[x] + 1, lca.dist[y] + 1});
            d--;
            query[i] = {x, lca.la(x, d), y, lca.la(y, d)};
        }
    } else if (treemode != "smart_bamboo"){
        assert(false);
    }

    dfs(0, -1);

    cout << n << '\n';
    for (int i = 0; i < n; ++i) {
        if (i > 0) {
            cout << pr[i] + 1;
        }
        if (i == n - 1) {
            cout << '\n';
        } else if (i > 0) {
            cout << ' ';
        }
    }
    for (int i = 0; i < n; i++) {
        cout << seg[i].first << " " << seg[i].second << '\n';
    }
    cout << q << '\n';
    for (int i = 0; i < q; i++) {
        for (int j = 0; j < 4; j++) {
            cout << query[i][j] + 1;
            if (j == 3) {
                cout << '\n';
            } else {
                cout << " ";
            }
        }
    }
}
