#include <iostream>
#include <set>
#include <algorithm>

#define MOD 1'000'000'007

int32_t bin_pow(int32_t base, int32_t power) {
    if(power == 0)
        return 1;
    if(power == 1)
        return base;
    int32_t prev = bin_pow(base, power / 2);
    prev = ((int64_t)prev * prev) % MOD;
    if(power % 2 == 1)
        prev = ((int64_t)prev * base) % MOD;
    return prev;
}

int32_t* parents;
int32_t* tin, *tout;
int32_t timer_ = 0;
void dfs(int32_t v, int32_t parent, std::vector<int32_t>* children) {
    tin[v] = timer_++;
    parents[v] = parent;
    for(int32_t i = 0; i < children[v].size(); i++)
        dfs(children[v][i], v, children);
    tout[v] = timer_;
}

bool is_ancestor(int32_t v1, int32_t v2) {
    return tin[v1] <= tin[v2] && tout[v1] >= tout[v2];
}

std::vector<int32_t> build_path(int32_t v1, int32_t v2) {
    std::vector<int32_t> ans, ans2;
    while(!is_ancestor(v1, v2)) {
        ans.push_back(v1);
        v1 = parents[v1];
    }
    ans.push_back(v1);
    while(!is_ancestor(v2, v1)) {
        ans2.push_back(v2);
        v2 = parents[v2];
    }
    for(int32_t i = (int32_t)ans2.size() - 1; i >= 0; i--)
        ans.push_back(ans2[i]);


    return ans;
}

struct DSU {
    int32_t* colors;
    std::pair<int32_t, int32_t>* segs;
    std::vector<int32_t>* per_color;

    DSU(int32_t n, std::pair<int32_t, int32_t>* arr) {
        colors = new int32_t[n];
        segs = new std::pair<int32_t, int32_t>[n];
        per_color = new std::vector<int32_t>[n];
        for(int32_t i = 0; i < n; i++) {
            colors[i] = i;
            segs[i] = arr[i];
            per_color[i] = {i};
        }
    }

    int32_t get_color(int32_t v) {
        return colors[v];
    }
};

int32_t* subtree_sizes;
void dfs_subtree_sizes(int32_t v, std::vector<int32_t>* children) {
    subtree_sizes[v] = 1;
    for(int32_t i = 0; i < children[v].size(); i++) {
        dfs_subtree_sizes(children[v][i], children);
        subtree_sizes[v] += subtree_sizes[children[v][i]];
    }
}

int32_t next_hld_path = 1;
void hld_reorder(int32_t v, int32_t parent, std::vector<int32_t>* children, int32_t* heavy_path_ind, std::vector<int32_t>& order, int32_t* inv_order) {
    std::sort(children[v].begin(), children[v].end(), [](int32_t v1, int32_t v2) {
        return subtree_sizes[v1] > subtree_sizes[v2];
    });
    order.push_back(v);
    inv_order[v] = order.size() - 1;
    for(int32_t i = 0; i < children[v].size(); i++) {
        heavy_path_ind[children[v][i]] = (i > 0) ? (next_hld_path++) : heavy_path_ind[v];
        hld_reorder(children[v][i], v, children, heavy_path_ind, order, inv_order);
    }
}

std::vector<std::pair<int32_t, int32_t> > build_hld_path(int32_t v1, int32_t v2, std::vector<int32_t>* heavy_paths, int32_t* heavy_path_ind) {
    std::vector<std::pair<int32_t, int32_t> > ans, ans2;
    while(true) {
        if(!is_ancestor(heavy_paths[heavy_path_ind[v1]][0], v2)) {
            // full path
            ans.emplace_back(v1, heavy_paths[heavy_path_ind[v1]][0]);
            v1 = parents[ans.back().second];
        } else {
            int32_t left = 0, right = heavy_paths[heavy_path_ind[v1]].size();
            while(right - left > 1) {
                int32_t m = (left + right) / 2;
                if(is_ancestor(heavy_paths[heavy_path_ind[v1]][m], v2))
                    left = m;
                else
                    right = m;
            }
            ans.emplace_back(v1, heavy_paths[heavy_path_ind[v1]][left]);
            v1 = ans.back().second;
            break;
        }
    }
    while(true) {
        if(!is_ancestor(heavy_paths[heavy_path_ind[v2]][0], v1)) {
            // full path
            ans2.emplace_back(v2, heavy_paths[heavy_path_ind[v2]][0]);
            v2 = parents[ans2.back().second];
        } else {
            if(v2 == v1)
                break;

            int32_t left = 0, right = heavy_paths[heavy_path_ind[v2]].size();
            while(right - left > 1) {
                int32_t m = (left + right) / 2;
                if(is_ancestor(heavy_paths[heavy_path_ind[v2]][m], v1))
                    left = m;
                else
                    right = m;
            }
            ans2.emplace_back(v2, heavy_paths[heavy_path_ind[v2]][right]);
            break;
        }
    }
    std::reverse(ans2.begin(), ans2.end());
    for(int32_t i = 0; i < ans2.size(); i++)
        ans.emplace_back(ans2[i].second, ans2[i].first);
    return ans;
}

std::vector<std::pair<int32_t, int32_t> > split_hld_path(std::vector<std::pair<int32_t, int32_t> >& hld_path, std::vector<int32_t>& pref_sums_merged) {
    int32_t ptr = 0;
    std::vector<std::pair<int32_t, int32_t> > hld_path_split;
    int32_t offset = 0;
    for(int32_t j = 0; j < pref_sums_merged.size(); j++) {
        int32_t to_split = pref_sums_merged[j] - (j == 0 ? 0 : pref_sums_merged[j - 1]);
        if(hld_path[ptr].first <= hld_path[ptr].second) {
            hld_path_split.emplace_back(hld_path[ptr].first + offset, hld_path[ptr].first + offset + to_split - 1);
            offset += to_split;
            if(hld_path_split.back().second == hld_path[ptr].second) {
                ptr++;
                offset = 0;
            }
        } else {
            hld_path_split.emplace_back(hld_path[ptr].first - offset, hld_path[ptr].first - offset - to_split + 1);
            offset += to_split;
            if(hld_path_split.back().second == hld_path[ptr].second) {
                ptr++;
                offset = 0;
            }
        }
    }
    return hld_path_split;
}

#define BASE 10'000'007
int32_t* base_powers;

struct SegtreeData {
    int32_t len;
    int32_t hash;
    int32_t inv_hash;
};

SegtreeData combine(SegtreeData d1, SegtreeData d2) {
    SegtreeData res;
    res.len = d1.len + d2.len;
    res.hash = ((int64_t)d1.hash * base_powers[d2.len] + d2.hash) % MOD;
    res.inv_hash = ((int64_t)d2.inv_hash * base_powers[d1.len] + d1.inv_hash) % MOD;
    return res;
}

struct Segtree {
    SegtreeData* data;

    Segtree(int32_t n) {
        data = new SegtreeData[4 * n];
        build(0, 0, n);
    }

    void build(int32_t node, int32_t left, int32_t right) {
        data[node] = {right - left, 0, 0};
        if(right - left == 1)
            return;
        int32_t m = (left + right) / 2;
        build(node * 2 + 1, left, m);
        build(node * 2 + 2, m, right);
    }

    SegtreeData get(int32_t node, int32_t left, int32_t right, int32_t query_left, int32_t query_right) {
        if(left >= query_right || right <= query_left)
            return {0, 0, 0};
        if(left >= query_left && right <= query_right)
            return data[node];
        int32_t m = (left + right) / 2;
        return combine(get(node * 2 + 1, left, m, query_left, query_right), get(node * 2 + 2, m, right, query_left, query_right));
    }

    int32_t get(int32_t node, int32_t left, int32_t right, int32_t query_left, int32_t query_right, bool direct) {
        if(direct)
            return get(node, left, right, query_left, query_right).hash;
        return get(node, left, right, query_left, query_right).inv_hash;
    }

    void update(int32_t node, int32_t left, int32_t right, int32_t ind, int32_t val) {
        if(left > ind || right <= ind)
            return;
        if(right - left == 1) {
            data[node].hash = val + 1;
            data[node].inv_hash = val + 1;
            return;
        }
        int32_t m = (left + right) / 2;
        update(node * 2 + 1, left, m, ind, val);
        update(node * 2 + 2, m, right, ind, val);
        data[node] = combine(data[node * 2 + 1], data[node * 2 + 2]);
    }
};

int main() {
    base_powers = new int32_t[200'005];
    base_powers[0] = 1;
    for(int32_t i = 1; i < 200'005; i++)
        base_powers[i] = ((int64_t)base_powers[i - 1] * BASE) % MOD;

    std::ios::sync_with_stdio(false);
    std::cin.tie(0);

    int32_t n;
    std::cin >> n;

    std::vector<int32_t>* children = new std::vector<int32_t>[n];
    for(int32_t i = 0; i < n - 1; i++) {
        int32_t parent;
        std::cin >> parent;
        parent--;

        children[parent].push_back(i + 1);
    }

    tin = new int32_t[n];
    tout = new int32_t[n];
    parents = new int32_t[n];
    dfs(0, -1, children);

    std::pair<int32_t, int32_t>* arr = new std::pair<int32_t, int32_t>[n];
    for(int32_t i = 0; i < n; i++)
        std::cin >> arr[i].first >> arr[i].second;

    subtree_sizes = new int32_t[n];
    dfs_subtree_sizes(0, children);

    std::vector<int32_t> order;
    int32_t* heavy_path_ind = new int32_t[n];
    int32_t* inv_order = new int32_t[n];
    heavy_path_ind[0] = 0;
    hld_reorder(0, -1, children, heavy_path_ind, order, inv_order);

    // Remake everything...
    int32_t* tin2 = new int32_t[n], *tout2 = new int32_t[n], *parents2 = new int32_t[n];
    int32_t* heavy_path_ind2 = new int32_t[n];
    std::vector<int32_t>* children2 = new std::vector<int32_t>[n];
    std::pair<int32_t, int32_t>* arr2 = new std::pair<int32_t, int32_t>[n];
    for(int32_t i = 0; i < n; i++) {
        tin2[i] = tin[order[i]];
        tout2[i] = tout[order[i]];
        parents2[i] = (parents[order[i]] == -1) ? -1 : inv_order[parents[order[i]]];
        heavy_path_ind2[i] = heavy_path_ind[order[i]];
        arr2[i] = arr[order[i]];
    }
    tin = tin2;
    tout = tout2;
    parents = parents2;
    //children = children2;
    arr = arr2;
    heavy_path_ind = heavy_path_ind2;

    std::vector<int32_t>* heavy_paths = new std::vector<int32_t>[next_hld_path];
    for(int32_t i = 0; i < n; i++)
        heavy_paths[heavy_path_ind[i]].push_back(i);

    int32_t m;
    std::cin >> m;

    int32_t ans = 1;
    for(int32_t i = 0; i < n; i++)
        ans = ((int64_t)ans * (arr[i].second - arr[i].first + 1)) % MOD;

    Segtree segtree(n);
    for(int32_t i = 0; i < n; i++)
        segtree.update(0, 0, n, i, i);

    DSU dsu(n, arr);
    for(int32_t i = 0; i < m; i++) {
        int32_t v1, v2, v3, v4;
        std::cin >> v1 >> v2 >> v3 >> v4;
        v1--;
        v2--;
        v3--;
        v4--;
        v1 = inv_order[v1];
        v2 = inv_order[v2];
        v3 = inv_order[v3];
        v4 = inv_order[v4];

        if(ans == 0) {
            std::cout << ans << "\n";
            continue;
        }

        std::vector<std::pair<int32_t, int32_t> > hld_path = build_hld_path(v1, v2, heavy_paths, heavy_path_ind);
        std::vector<std::pair<int32_t, int32_t> > hld_path2 = build_hld_path(v3, v4, heavy_paths, heavy_path_ind);

        std::vector<int32_t> pref_sums1, pref_sums2;
        int32_t sum = 0;
        for(int32_t j = 0; j < hld_path.size(); j++) {
            sum += std::abs(hld_path[j].first - hld_path[j].second) + 1;
            pref_sums1.push_back(sum);
        }
        sum = 0;
        for(int32_t j = 0; j < hld_path2.size(); j++) {
            sum += std::abs(hld_path2[j].first - hld_path2[j].second) + 1;
            pref_sums2.push_back(sum);
        }
        std::vector<int32_t> pref_sums_merged;
        pref_sums_merged.resize(pref_sums1.size() + pref_sums2.size());
        std::merge(pref_sums1.begin(), pref_sums1.end(), pref_sums2.begin(), pref_sums2.end(), pref_sums_merged.begin());
        pref_sums_merged.resize(std::unique(pref_sums_merged.begin(), pref_sums_merged.end()) - pref_sums_merged.begin());

        std::vector<std::pair<int32_t, int32_t> > hld_path_split = split_hld_path(hld_path, pref_sums_merged);
        std::vector<std::pair<int32_t, int32_t> > hld_path2_split = split_hld_path(hld_path2, pref_sums_merged);
        hld_path = hld_path_split;
        hld_path2 = hld_path2_split;

        // --------------
        for(int32_t j = 0; j < hld_path_split.size() && ans > 0; j++) {
            while(segtree.get(0, 0, n, std::min(hld_path[j].first, hld_path[j].second),
                                       std::max(hld_path[j].first, hld_path[j].second) + 1,
                                       hld_path[j].first < hld_path[j].second) !=
                                       segtree.get(0, 0, n, std::min(hld_path2[j].first, hld_path2[j].second),
                                                   std::max(hld_path2[j].first, hld_path2[j].second) + 1,
                                                   hld_path2[j].first < hld_path2[j].second)) {
                int32_t left = 0, right = std::abs(hld_path[j].first - hld_path[j].second) + 1;
                while(right - left > 1) {
                    int32_t mid = (left + right) / 2;
                    int32_t hash1;
                    if(hld_path[j].first < hld_path[j].second)
                        hash1 = segtree.get(0, 0, n, hld_path[j].first, hld_path[j].first + mid, true);
                    else
                        hash1 = segtree.get(0, 0, n, hld_path[j].first - mid + 1, hld_path[j].first + 1, false);
                    int32_t hash2;
                    if(hld_path2[j].first < hld_path2[j].second)
                        hash2 = segtree.get(0, 0, n, hld_path2[j].first, hld_path2[j].first + mid, true);
                    else
                        hash2 = segtree.get(0, 0, n, hld_path2[j].first - mid + 1, hld_path2[j].first + 1, false);

                    if(hash1 == hash2)
                        left = mid;
                    else
                        right = mid;
                }

                int32_t pv = hld_path[j].first < hld_path[j].second ? hld_path[j].first + left : hld_path[j].first - left;
                int32_t qv = hld_path2[j].first < hld_path2[j].second ? hld_path2[j].first + left : hld_path2[j].first - left;

                int32_t p = dsu.get_color(pv);
                int32_t q = dsu.get_color(qv);

                int32_t new_left = std::max(dsu.segs[p].first, dsu.segs[q].first);
                int32_t new_right = std::min(dsu.segs[p].second, dsu.segs[q].second);
                if(new_right < new_left) {
                    ans = 0;
                    break;
                }

                ans = ((int64_t)ans * bin_pow(dsu.segs[p].second - dsu.segs[p].first + 1, MOD - 2)) % MOD;
                ans = ((int64_t)ans * bin_pow(dsu.segs[q].second - dsu.segs[q].first + 1, MOD - 2)) % MOD;
                ans = ((int64_t)ans * (new_right - new_left + 1)) % MOD;

                // join p ans q
                if(dsu.per_color[p].size() < dsu.per_color[q].size())
                    std::swap(p, q);

                dsu.segs[p] = {new_left, new_right};
                for(int32_t v : dsu.per_color[q]) {
                    dsu.colors[v] = p;
                    dsu.per_color[p].push_back(v);
                    segtree.update(0, 0, n, v, p);
                }
                dsu.per_color[q].clear();
            }
        }

        std::cout << ans << "\n";
    }
    return 0;
}
/*
 5
 1 1 2 2
 1 10
 1 100
 1 1000
 1 10000
 1 100000
 5
 1 1 2 2
 1 2 2 1
 3 4 4 3
 3 4 3 5
 5 5 4 4
 */



