#include <map>
#include <set>
#include <array>
#include <cmath>
#include <deque>
#include <bitset>
#include <random>
#include <string>
#include <vector>
#include <cassert>
#include <complex>
#include <iomanip>
#include <iostream>
#include <algorithm>
#include <unordered_map>
#include <unordered_set>

using namespace std;

const int MAXN = 200200;
const int MOD = 1e9 + 7;

int add(int a, int b) {
  if (a + b < MOD)
    return a + b;
  return a + b - MOD;
}

int sub(int a, int b) {
  if (a >= b)
    return a - b;
  return a + MOD - b;
}

int mul(int a, int b) {
  return a * 1ll * b % MOD;
}

int pw(int a, int n) {
  int ans = 1;
  while (n) {
    if (n & 1)
      ans = mul(ans, a);
    n >>= 1;
    a = mul(a, a);
  }
  return ans;
}

const unsigned long long BASE1 = 207371;
unsigned long long BASEPOW1[MAXN];

struct Hash{
  unsigned long long h1;
  int len;
  Hash(unsigned long long h1, int len): h1(h1), len(len) {}
  Hash(): h1(0), len(0) {}
};

bool operator ==(const Hash& first, const Hash& second) {
  return first.h1 == second.h1;
}

Hash combine(const Hash& a, const Hash& b) {
  return Hash(a.h1 * BASEPOW1[b.len] + b.h1, a.len + b.len);
}

const int K = 18;

vector<int> G[MAXN];
int depth[MAXN];
int sz[MAXN];

void dfs1(int v, int w) {
  depth[v] = w;
  sz[v] = 1;
  for (int i = 0; i < G[v].size(); ++i) {
    dfs1(G[v][i], w + 1);
    sz[v] += sz[G[v][i]];
    if (sz[G[v][i]] > sz[G[v][0]])
      swap(G[v][i], G[v][0]);
  }
}

vector<int> order;
int tin[MAXN];
int path_id[MAXN];
int path_st[MAXN];

void dfs2(int v) {
  tin[v] = order.size();
  order.emplace_back(v);
  if (G[v].empty()) {
    path_id[v] = v;
    path_st[path_id[v]] = v;
    return;
  }
  for (int u : G[v]) {
    dfs2(u);
  }
  path_id[v] = path_id[G[v][0]];
  path_st[path_id[v]] = v;
}

int up[MAXN][K];

int la(int v, int d) {
  for (int k = K - 1; k >= 0; --k) {
    if (d >= (1 << k)) {
      d -= 1 << k;
      v = up[v][k];
    }
  }
  return v;
}

int lca(int u, int v) {
  if (depth[u] > depth[v])
    u = la(u, depth[u] - depth[v]);
  else
    v = la(v, depth[v] - depth[u]);
  if (u == v)
    return u;
  for (int k = K - 1; k >= 0; --k) {
    if (up[v][k] != up[u][k]) {
      v = up[v][k];
      u = up[u][k];
    }
  }
  return up[v][0];
}

int cur_ans = 1;
int id[MAXN];
vector<int> sets[MAXN];
pair<int, int> segm[MAXN];

const int N = 1 << 18;
Hash hashes[2 * N + 228];
Hash hashes_rev[2 * N + 228];

void build(int n) {
  for (int i = 0; i < n; ++i)
    hashes[N + i] = hashes_rev[N + i] = Hash(order[i], 1);
  fill(hashes + N + n, hashes + 2 * N, Hash());
  fill(hashes_rev + N + n, hashes_rev + 2 * N, Hash());
  for (int i = N - 1; i > 0; --i) {
    hashes[i] = combine(hashes[2 * i], hashes[2 * i + 1]);
    hashes_rev[i] = combine(hashes_rev[2 * i + 1], hashes_rev[2 * i]);
  }
}

void upd(int i, int x) {
  i += N;
  hashes[i] = hashes_rev[i] = Hash(x, 1);
  while (i > 1) {
    i >>= 1;
    hashes[i] = combine(hashes[2 * i], hashes[2 * i + 1]);
    hashes_rev[i] = combine(hashes_rev[2 * i + 1], hashes_rev[2 * i]);
  }
}

Hash get(int left, int right) {
  --right;
  Hash ans_l, ans_r;
  left += N;
  right += N;
  while (left < right) {
    if (left & 1)
      ans_l = combine(ans_l, hashes[left]);
    if (!(right & 1))
      ans_r = combine(hashes[right], ans_r);
    left = (left + 1) >> 1;
    right = (right - 1) >> 1;
  }
  if (left == right)
    ans_l = combine(ans_l, hashes[left]);
  return combine(ans_l, ans_r);
}

Hash get_rev(int left, int right) {
  --right;
  Hash ans_l, ans_r;
  left += N;
  right += N;
  while (left < right) {
    if (left & 1)
      ans_l = combine(hashes_rev[left], ans_l);
    if (!(right & 1))
      ans_r = combine(ans_r, hashes_rev[right]);
    left = (left + 1) >> 1;
    right = (right - 1) >> 1;
  }
  if (left == right)
    ans_l = combine(hashes_rev[left], ans_l);
  return combine(ans_r, ans_l);
}

int n;

void merge(int v, int u) {
  //cerr << "merge " << v << ' ' << u << '\n';
  v = id[v];
  u = id[u];
  if (v == u)
    return;
  if (sets[u].size() > sets[v].size())
    swap(u, v);
  sets[v].reserve(sets[v].size() + sets[u].size());
  for (int x : sets[u]) {
    id[x] = v;
    upd(tin[x], v);
    sets[v].emplace_back(x);
  }
  sets[u].clear();
  cur_ans = mul(cur_ans, pw(max(0, segm[u].second - segm[u].first + 1), MOD - 2));
  cur_ans = mul(cur_ans, pw(max(0, segm[v].second - segm[v].first + 1), MOD - 2));
  segm[v].first = max(segm[v].first, segm[u].first);
  segm[v].second = min(segm[v].second, segm[u].second);
  cur_ans = mul(cur_ans, max(0, segm[v].second - segm[v].first + 1));
}

void make_path(int a, int b, vector<pair<pair<int, int>, bool>>& path1) {
  int l1 = lca(a, b);
  while (depth[path_st[path_id[a]]] > depth[l1]) {
    path1.emplace_back(make_pair(a, depth[a] - depth[path_st[path_id[a]]] + 1), false);
    a = up[path_st[path_id[a]]][0];
  }
  path1.emplace_back(make_pair(a, depth[a] - depth[l1] + 1), false);
  int sz = path1.size();
  while (depth[path_st[path_id[b]]] > depth[l1]) {
    path1.emplace_back(make_pair(b, depth[b] - depth[path_st[path_id[b]]] + 1), true);
    b = up[path_st[path_id[b]]][0];
  }
  if (b != l1)
    path1.emplace_back(make_pair(b, depth[b] - depth[l1]), true);
  if (sz != path1.size())
    reverse(path1.begin() + sz, path1.end());
}

Hash get_hash(int i, bool rev, int from, int to, int len) {
  if (rev)
    return get(i - len + 1 + from, i - len + 1 + to);
  return get_rev(i - to + 1, i - from + 1);
}

int at_point(int i, bool rev, int j, int len) {
  return rev ? order[i - len + 1 + j] : order[i - j];
}

void try_merging(int i1, bool rev1, int i2, bool rev2, int len) {
  //cerr << "try " << i1 << ' ' << rev1 << ' ' << i2 << ' ' << rev2 << ' ' << len << '\n';
  if (len < 30) {
    for (int j = 0; j < len; ++j)
      merge(at_point(i1, rev1, j, len), at_point(i2, rev2, j, len));
    return;
  }
  int left = 0;
  while (left < len) {
    if (get_hash(i1, rev1, left, len, len) == get_hash(i2, rev2, left, len, len))
      return;
    int right = len;
    while (right - left > 41) {
      int mid = (left + right) / 2;
      if (get_hash(i1, rev1, left, mid, len) == get_hash(i2, rev2, left, mid, len))
        left = mid;
      else
        right = mid;
    }
    for (int j = left; j < right; ++j)
      merge(at_point(i1, rev1, j, len), at_point(i2, rev2, j, len));
    left = right;
  }
}

int main() {
#ifdef DEBUG
  freopen("input.txt", "r", stdin);
  freopen("output.txt", "w", stdout);
#else
  ios::sync_with_stdio(false);
  cin.tie(0);
  cout.tie(0);
#endif

  BASEPOW1[0] = 1;
  for (int i = 1; i < MAXN; ++i)
    BASEPOW1[i] = BASEPOW1[i - 1] * BASE1;

  cin >> n;
  for (int i = 1; i < n; ++i) {
    int p;
    cin >> p;
    up[i][0] = p - 1;
    G[p - 1].emplace_back(i);
  }
  dfs1(0, 0);
  dfs2(0);
  for (int j = 1; j < K; ++j) {
    for (int i = 0; i < n; ++i)
      up[i][j] = up[up[i][j - 1]][j - 1];
  }
  for (int i = 0; i < n; ++i) {
    id[i] = i;
    sets[i].emplace_back(i);
    int l, r;
    cin >> l >> r;
    segm[i] = make_pair(l, r);
    cur_ans = mul(cur_ans, r - l + 1);
  }
  build(n);
  int q;
  cin >> q;
  vector<pair<pair<int, int>, bool>> path1;
  vector<pair<pair<int, int>, bool>> path2;
  while (q--) {
    int a, b, c, d;
    cin >> a >> b >> c >> d;
    --a; --b; --c; --d;
    make_path(a, b, path1);
    make_path(c, d, path2);
    int j = 0;
    int d2 = 0;

    for (int i = 0; i < path1.size(); ++i) {
      int d1 = 0;
      while (d1 < path1[i].first.second) {
        int now = min(path1[i].first.second - d1, path2[j].first.second - d2);
        try_merging(tin[path1[i].first.first] - (path1[i].second ? path1[i].first.second - d1 - now : d1), path1[i].second,
                    tin[path2[j].first.first] - (path2[j].second ? path2[j].first.second - d2 - now : d2), path2[j].second, now);
        d1 += now;
        d2 += now;
        if (d2 == path2[j].first.second) {
          ++j;
          d2 = 0;
        }
      }
    }
    path1.clear();
    path2.clear();
    cout << cur_ans << '\n';
  }
  return 0;
}
