#include "testlib.h"
#include "helper.h"
#include <iostream>
#include <vector>
#include <map>
#include <set>
#include <algorithm>

using namespace std;

#define fi first
#define se second
#define all(x) (x).begin(), (x).end()
#define pb emplace_back
#define ll long long

map<string, string> params;

long long get(string name, long long def = 0) {
    if (params.count(name)) {
        return stoll(params[name]);
    } else {
        return def;
    }
}

const int MAXT = 40'000;
const int MAXN = 200'000;
const int MAXM = 200'000;
const int MAXC = (1 << 30) - 1;

void dfs(int v, vector<vector<int>> &g, vector<bool> &usd, vector<int> &b) {
    usd[v] = true;
    b.pb(v);
    for (auto &to : g[v]) {
        if (!usd[to]) dfs(to, g, usd, b);
    }
}

int main(int argc, char* argv[]) {
    registerGen(argc, argv, 1);
    for (int i = 1; i < argc; i++) {
        string s = string(argv[i]);
        if (s.find("=") != string::npos) {
            auto pos = s.find("=");
            params[s.substr(0, pos)] = s.substr(pos + 1);
        } else {
            params[s] = "";
        }
    }
    int t = get("t", MAXT);
    int maxc = get("maxc", MAXC);
    int coef = get("coef", 0);
    int elongation = get("elongation", 1);
    vector<int> n, m;
    if (coef == 0) {
        n = rnd.partition(t, MAXN, 2);
        m = rnd.partition(t, MAXM, 1);
    } else if (coef > 0) {
        int maxn = MAXM / coef;
        n = rnd.partition(t, maxn, 2);
        m.resize(t);
        for (int i = 0; i < t; i++) m[i] = coef * n[i];
    }
    cout << t << "\n";
    for (int i = 0; i < t; i++) {
        cout << n[i] << " " << m[i] << "\n";
        int k = m[i] - min(n[i] - 1, m[i]);
        vector<int> a = randomSet(n[i], min(n[i], m[i] + 1));
        Graph G;
        for (int j = 1; j < a.size(); j++) {
            int u = a[rnd.wnext(0, j - 1, -elongation)];
            int v = a[j];
            int w = rnd.next(0, maxc);
            G.AddEdge(u, v, w);
        }

        for (int j = 0; j < k; j++) {
            int u = rnd.next(0, n[i] - 1);
            while (u == n[i] - 1) {
                u = rnd.next(0, n[i] - 1);
            }
            int v = rnd.wnext(u + 1, n[i] - 1, elongation);
            int w = rnd.next(0, maxc);
            G.AddEdge(u, v, w);
        }

        G.Print();
        int s = rnd.next(0, n[i] - 1);
        vector<vector<int>> g(n[i]);
        for (auto &e : G.e) {
            g[e.u].pb(e.v);
        }
        vector<int> b;
        vector<bool> usd(n[i]);
        dfs(s, g, usd, b);
        set<int> r;
        for (auto &c : b) {
            r.insert(c);
        }
        vector<int> ur;
        for (int v = 0; v < n[i]; v++) {
            if (r.find(v) == r.end()) {
                ur.pb(v);
            }
        }
        int t = b[rnd.next(0, (int)b.size() - 1)];
        if (rnd.next(1, 100) == 1) {
            t = s;
        } else if (rnd.next(1, 100) == 2 && !ur.empty()) {
            t = ur[rnd.next(0, (int)ur.size() - 1)];
        }
        cout << s + 1 << " " << t + 1 << "\n";
    }
    return 0;
}
