#include "testlib.h"
#include "helper.h"
#include <iostream>
#include <vector>
#include <map>
#include <set>
#include <algorithm>

using namespace std;

#define fi first
#define se second
#define all(x) (x).begin(), (x).end()
#define pb emplace_back
#define ll long long
#define pii pair<int, int>

map<string, string> params;

long long get(string name, long long def = 0) {
    if (params.count(name)) {
        return stoll(params[name]);
    } else {
        return def;
    }
}

const int MAXT = 40'000;
const int MAXN = 200'000;
const int MAXM = 200'000;
const int MAXC = (1 << 30) - 1;

void dfs(int v, vector<vector<int>> &g, vector<bool> &usd, vector<int> &b) {
    usd[v] = true;
    b.pb(v);
    for (auto &to : g[v]) {
        if (!usd[to]) dfs(to, g, usd, b);
    }
}

vector<pii> gen1(vector<int> id) { // just a cycle
    shuffle(id.begin(), id.end());
    vector<pii> edges;
    for (int i = 0; i < id.size(); i++) {
        edges.pb(id[i], id[(i + 1) % id.size()]);
    }
    return edges;
}

vector<pii> gen2(vector<int> id) { // some union of cycles
    shuffle(id.begin(), id.end());
    int n = id.size();
    vector<pii> edges;
    vector<vector<int>> cycles(n);
    for (int i = 0; i < n; i++) {
        cycles[i].pb(id[i]);
    }
    while (cycles.size() > 1) {
        vector<int> now;
        for (int i = 0; i < cycles.size(); i++) {
            now.pb(i);
        }
        shuffle(all(now));
        int k = rnd.next(2, (int)cycles.size());
        map<int, int> used;
        vector<int> cur;
        for (int i = 0; i < k; i++) {
            used[now[i]] = 1;
            for (auto &v : cycles[now[i]]) {
                cur.pb(v);
            }
            int a = cycles[now[i]][rnd.next(0, (int)cycles[now[i]].size() - 1)];
            int b = cycles[now[(i + 1) % k]][rnd.next(0, (int)cycles[now[(i + 1) % k]].size() - 1)];
            edges.pb(a, b);
        }
        vector<vector<int>> new_cycles;
        for (int i = 0; i < cycles.size(); i++) {
            if (used[i]) {
                continue;
            }
            new_cycles.pb(cycles[i]);
        }
        new_cycles.pb(cur);
        cycles = new_cycles;
    }
    return edges;
}

vector<pii> gen3(vector<int> id) { // tree-like
    shuffle(id.begin(), id.end());
    vector<pii> edges;
    int n = id.size();
    for (int i = 1; i < n; i++) {
        edges.pb(id[rnd.next(0, i - 1)], id[i]);
    }
    for (int i = 1; i < n; i++) {
        edges.pb(id[i], id[rnd.next(0, i - 1)]);
    }
    return edges;
}

void dfs(int v, vector<vector<pair<int, int>>> &g, vector<bool> &usd) {
    usd[v] = true;
    for (auto &to: g[v]) {
        if (!usd[to.fi]) {
            dfs(to.fi, g, usd);
        }
    }
}

void dfs2(int v, vector<vector<int>> &g, vector<bool> &usd, vector<int> &a) {
    usd[v] = 1;
    for (auto &to: g[v]) {
        if (!usd[to]) {
            dfs2(to, g, usd, a);
        }
    }
    a.pb(v);
}

void scc(int n, vector<vector<int>> &g, vector<bool> &cycle) {
    vector<vector<int>> gr(n);
    for (int v = 0; v < n; v++) {
        for (auto &to: g[v]) {
            gr[to].pb(v);
        }
    }
    vector<int> ts;
    vector<bool> usd(n);
    for (int v = 0; v < n; v++) {
        if (!usd[v]) {
            dfs2(v, g, usd, ts);
        }
    }
    reverse(all(ts));
    usd.assign(n, false);
    for (auto &v: ts) {
        if (!usd[v]) {
            vector<int> comp;
            dfs2(v, gr, usd, comp);
            if (comp.size() > 1) {
                for (auto &u: comp) {
                    cycle[u] = true;
                }
            } else {
                bool loop = false;
                for (auto &to: g[v]) {
                    if (to == v) {
                        loop = true;
                    }
                }
                if (loop) {
                    cycle[v] = true;
                }
            }
        }
    }
}

void dfs3(int v, vector<vector<pair<int, int>>> &g, vector<bool> &processed, vector<bool> &cycle) {
    if (processed[v]) {
        return;
    }
    processed[v] = true;
    if (cycle[v]) {
        return;
    }
    for (auto &e: g[v]) {
        dfs3(e.first, g, processed, cycle);
        if (cycle[e.first]) {
            cycle[v] = true;
            return;
        }
    }
}

vector<bool> solve(int n, int m, const vector<pii> &e, int s, int t) {
    vector<vector<pair<int, int>>> g(n);
    vector<vector<pair<int, int>>> gr(n);
    for (int it = 0; it < m; it++) {
        int v = e[it].first;
        int u = e[it].second;
        int w = 0;
        g[v].pb(u, w);
        gr[u].pb(v, w);
    }
    vector<bool> fromS(n), fromT(n);
    dfs(s, g, fromS);
    dfs(t, gr, fromT);
    vector<vector<int>> g1(n);
    for (int v = 0; v < n; v++) {
        for (auto &e: g[v]) {
            g1[v].pb(e.first);
        }
    }
    vector<bool> cycle(n);
    scc(n, g1, cycle);
    vector<bool> processed_before(n);
    vector<bool> processed_after(n);
    vector<bool> cycle_before(n);
    vector<bool> cycle_after(n);
    for (int i = 0; i < n; i++) {
        if (cycle[i]) {
            cycle_after[i] = cycle_before[i] = true;
        }
    }
    for (int v = 0; v < n; v++) {
        if (!processed_after[v]) {
            dfs3(v, g, processed_after, cycle_after);
        }
        if (!processed_before[v]) {
            dfs3(v, gr, processed_before, cycle_before);
        }
    }
    for (int i = 0; i < n; i++) {
        if (cycle_before[i] || cycle_after[i]) {
            cycle[i] = true;
        }
    }
    for (int i = 0; i < n; i++) {
        if (!fromS[i] || !fromT[i]) {
            cycle[i] = false;
        }
    }
    return cycle;
}

int main(int argc, char* argv[]) {
    registerGen(argc, argv, 1);
    for (int i = 1; i < argc; i++) {
        string s = string(argv[i]);
        if (s.find("=") != string::npos) {
            auto pos = s.find("=");
            params[s.substr(0, pos)] = s.substr(pos + 1);
        } else {
            params[s] = "";
        }
    }
    int t = get("t", MAXT);
    int maxc = get("maxc", MAXC);
    int maxm = get("maxm", MAXN);
    bool ok = get("ok", 1);
    int coef = get("coef", 5);
    int comp = get("comp", -100);
    vector<int> ns = rnd.partition(t, maxm / coef, 1);
    cout << t << "\n";
    for (int test_case = 0; test_case < t; test_case++) {
        int n = ns[test_case];
        int m = n * coef;
        cout << n << " " << m << "\n";
        vector<int> a(n);
        for (int i = 0; i < n; i++) {
            a[i] = i;
        }
        shuffle(a.begin(), a.end());

        // creating scc
        int k = rnd.wnext(1, n, -3);
        int cnt_comp = rnd.wnext(1, k, comp);
        vector<int> sizes = rnd.partition(cnt_comp, k, 1);
        vector<vector<int>> comps;
        vector<int> all_comps;
        for (int i = 0; i < cnt_comp; i++) {
            vector<int> now;
            while (now.size() < sizes[i]) {
                now.pb(a.back());
                all_comps.pb(a.back());
                a.pop_back();
            }
            comps.pb(now);
        }
        vector<pii> edges;

        for (int i = 0; i < cnt_comp; i++) {
            vector<int> id = comps[i];
            vector<pii> new_edges;
            int type = rnd.next(1, 3);
            if (type == 1) {
                new_edges = gen1(id);
            } else if (type == 2) {
                new_edges = gen2(id);
            } else {
                new_edges = gen3(id);
            }
            while (new_edges.size() < id.size() * coef) {
                new_edges.pb(id[rnd.next(0, (int)id.size() - 1)], id[rnd.next(0, (int)id.size() - 1)]);
            }
            for (auto &e : new_edges) {
                edges.pb(e);
            }
        }


        int s = rnd.next(0, n - 1);
        int t = rnd.next(0, n - 1);

        // creating dag
        vector<int> dag;
        k = rnd.wnext(0, (int)a.size(), -3);
        dag.pb(s);
        for (int i = 0; i < k; i++) {
            dag.pb(a.back());
            a.pop_back();
        }
        dag.pb(t);
        for (int it = 0; it < 10 * coef; it++) {
            for (int i = 1; i < (int)dag.size(); i++) {
                edges.pb(dag[rnd.next(0, i - 1)], dag[i]);
            }
        }

        // adding edges between scc
        for (int it = 0; it < coef; it++) {
            if (comps.empty()) {
                break;
            }
            int id1 = rnd.next(0, (int)comps.size() - 1);
            int id2 = rnd.next(0, (int)comps.size() - 1);
            edges.pb(comps[id1][rnd.next(0, (int)comps[id1].size() - 1)], comps[id2][rnd.next(0, (int)comps[id2].size() - 1)]);
        }

        // adding trash edges
        for (int it = 0; it < coef; it++) {
            for (int i = 0; i < n; i++) {
                int type = rnd.next(0, 1);
                if (type == 0 && !all_comps.empty()) {
                    int v = all_comps[rnd.next(0, (int)all_comps.size() - 1)];
                    edges.pb(v, v);
                }  else if (!all_comps.empty()) {
                    int v = all_comps[rnd.next(0, (int)all_comps.size() - 1)];
                    int u = all_comps[rnd.next(0, (int)all_comps.size() - 1)];
                    edges.pb(u, v);
                }
            }
        }

        // deleting extra edges
        while (edges.size() > m) {
            edges.pop_back();
        }

        // finding "bad" vertices
        vector<bool> cycle = solve(n, m, edges, s, t);

        // xor assignment
        vector<int> w(n);
        for (int i = 0; i < n; i++) {
            w[i] = rnd.next(0, maxc);
        }
        w[s] = 0;
        w[t] = 0;
        if (!ok && rnd.next(0, 2) == 0) {
            w[t] = rnd.next(0, maxc);
        }

        // graph building
        Graph G;
        for (auto &e : edges) {
            if (cycle[e.first] && cycle[e.second]) {
                int cur = w[e.first] ^ w[e.second];
                if (ok == 0) {
                    if (rnd.next(0, m / 10) == 0) {
                        cur = rnd.next(0, maxc);
                    }
                }
                G.AddEdge(e.first, e.second, cur);
            } else {
                G.AddEdge(e.first, e.second, rnd.next(0, maxc));
            }
        }

        G.Print();
        cout << s + 1 << " " << t + 1 << "\n";
    }
    return 0;
}
