#include "testlib.h"
#include "helper.h"
#include <iostream>
#include <vector>
#include <map>
#include <set>
#include <algorithm>
#include <cassert>

using namespace std;

#define fi first
#define se second
#define all(x) (x).begin(), (x).end()
#define pb emplace_back
#define ll long long

map<string, string> params;

long long get(string name, long long def = 0) {
    if (params.count(name)) {
        return stoll(params[name]);
    } else {
        return def;
    }
}

const int MAXT = 40'000;
const int MAXN = 200'000;
const int MAXM = 200'000;
const int MAXC = (1 << 30) - 1;

void dfs(int v, vector<vector<int>> &g, vector<bool> &usd, vector<int> &b) {
    usd[v] = true;
    b.pb(v);
    for (auto &to : g[v]) {
        if (!usd[to]) dfs(to, g, usd, b);
    }
}

int main(int argc, char* argv[]) {
    registerGen(argc, argv, 1);
    for (int i = 1; i < argc; i++) {
        string s = string(argv[i]);
        if (s.find("=") != string::npos) {
            auto pos = s.find("=");
            params[s.substr(0, pos)] = s.substr(pos + 1);
        } else {
            params[s] = "";
        }
    }
    int t = get("t", 1);
    int sumn = get("n", MAXN);
    int summ = get("m", MAXM);
    int maxc = get("maxc", MAXC);
    int mode = get("mode", 0);
    auto N = rnd.partition(t, sumn, 50);
    cout << t << "\n";
    for (int tc = 0; tc < t; tc++) {
        int n = N[tc], m = N[tc];
        cout << n << " " << m << "\n";
        if (mode == 0) {
            vector<int> allv(n);
            for (int i = 0; i < n; i++) allv[i] = i;
            shuffle(all(allv));
            int path_size = rnd.next(3, n / 2);
            vector<int> path;
            for (int i = 0; i < path_size; i++) {
                path.pb(allv.back());
                allv.pop_back();
            }
            int a = path[0];
            int b = path.back();
            int v = path[rnd.next(0, (int) path.size() - 2)];
            assert(v != b);
            int path2_size = rnd.next(2, (int) allv.size() - 1);
            vector<int> path2;
            for (int i = 0; i < path2_size; i++) {
                path2.pb(allv.back());
                allv.pop_back();
            }
            vector<int> cycle = allv;
            vector<int> cycle_vals(cycle.size());
            int cycle_xor = 0;
            do {
                cycle_xor = 0;
                for (int i = 0; i < cycle.size(); i++) {
                    cycle_vals[i] = rnd.next(0, maxc);
                    cycle_xor ^= cycle_vals[i];
                }
            } while (cycle_xor == 0);
            Graph G;
            for (int i = 0; i < cycle.size(); i++) {
                G.AddEdge(cycle[i], cycle[(i + 1) % cycle.size()], cycle_vals[i]);
            }
            vector<int> path_vals(path.size() - 1);
            int path_xor = 0;
            do {
                path_xor = 0;
                for (int i = 0; i < path_vals.size(); i++) {
                    path_vals[i] = rnd.next(0, maxc);
                    path_xor ^= path_vals[i];
                }
            } while (path_xor == 0);
            for (int i = 0; i < path.size() - 1; i++) {
                G.AddEdge(path[i], path[i + 1], path_vals[i]);
            }
            G.AddEdge(v, path2[0], rnd.next(0, maxc));
            for (int i = 0; i + 1 < path2.size(); i++) {
                G.AddEdge(path2[i], path2[i + 1], rnd.next(0, maxc));
            }
            G.AddEdge(path2.back(), cycle[0], rnd.next(0, maxc));
            G.Print();
            cout << a + 1 << " " << b + 1 << "\n";
        } else if (mode == 1) {
            vector<int> allv(n);
            for (int i = 0; i < n; i++) allv[i] = i;
            shuffle(all(allv));
            int path_size = rnd.next(3, n / 2);
            vector<int> path;
            for (int i = 0; i < path_size; i++) {
                path.pb(allv.back());
                allv.pop_back();
            }
            int a = path[0];
            int b = path.back();
            int v = path[rnd.next(1, (int) path.size() - 1)];
            assert(v != a);
            int path2_size = rnd.next(2, (int) allv.size() - 1);
            vector<int> path2;
            for (int i = 0; i < path2_size; i++) {
                path2.pb(allv.back());
                allv.pop_back();
            }
            vector<int> cycle = allv;
            vector<int> cycle_vals(cycle.size());
            int cycle_xor = 0;
            do {
                cycle_xor = 0;
                for (int i = 0; i < cycle.size(); i++) {
                    cycle_vals[i] = rnd.next(0, maxc);
                    cycle_xor ^= cycle_vals[i];
                }
            } while (cycle_xor == 0);
            Graph G;
            for (int i = 0; i < cycle.size(); i++) {
                G.AddEdge(cycle[i], cycle[(i + 1) % cycle.size()], cycle_vals[i]);
            }
            vector<int> path_vals(path.size() - 1);
            int path_xor = 0;
            do {
                path_xor = 0;
                for (int i = 0; i < path_vals.size(); i++) {
                    path_vals[i] = rnd.next(0, maxc);
                    path_xor ^= path_vals[i];
                }
            } while (path_xor == 0);
            for (int i = 0; i < path.size() - 1; i++) {
                G.AddEdge(path[i], path[i + 1], path_vals[i]);
            }
            G.AddEdge(path2[0], v, rnd.next(0, maxc));
            for (int i = 0; i + 1 < path2.size(); i++) {
                G.AddEdge(path2[i], path2[i + 1], rnd.next(0, maxc));
            }
            G.AddEdge(cycle[0], path2.back(), rnd.next(0, maxc));
            G.Print();
            cout << a + 1 << " " << b + 1 << "\n";
        } else if (mode == 2) {
            vector<int> allv(n);
            for (int i = 0; i < n; i++) allv[i] = i;
            shuffle(all(allv));
            int path_size = rnd.next(3, n / 2);
            vector<int> path;
            for (int i = 0; i < path_size; i++) {
                path.pb(allv.back());
                allv.pop_back();
            }
            int a = path[0];
            int b = path.back();
            int v = path[rnd.next(1, (int) path.size() - 2)];
            assert(v != a && v != b);
            vector<int> cycle = allv;
            cycle.pb(v);
            vector<int> cycle_vals(cycle.size());
            int cycle_xor = 0;
            do {
                cycle_xor = 0;
                for (int i = 0; i < cycle.size(); i++) {
                    cycle_vals[i] = rnd.next(0, maxc);
                    cycle_xor ^= cycle_vals[i];
                }
            } while (cycle_xor == 0);
            Graph G;
            for (int i = 0; i < cycle.size(); i++) {
                G.AddEdge(cycle[i], cycle[(i + 1) % cycle.size()], cycle_vals[i]);
            }
            vector<int> path_vals(path.size() - 1);
            int path_xor = 0;
            do {
                path_xor = 0;
                for (int i = 0; i < path_vals.size(); i++) {
                    path_vals[i] = rnd.next(0, maxc);
                    path_xor ^= path_vals[i];
                }
            } while (path_xor == 0);
            for (int i = 0; i < path.size() - 1; i++) {
                G.AddEdge(path[i], path[i + 1], path_vals[i]);
            }
            G.Print();
            cout << a + 1 << " " << b + 1 << "\n";
        } else if (mode == 3) {
            vector<int> allv(n);
            for (int i = 0; i < n; i++) allv[i] = i;
            shuffle(all(allv));
            int path_size = rnd.next(3, n / 2);
            vector<int> path;
            for (int i = 0; i < path_size; i++) {
                path.pb(allv.back());
                allv.pop_back();
            }
            int a = path[0];
            int b = path.back();
            int v = path[rnd.next(1, (int) path.size() - 2)];
            assert(v != a && v != b);
            vector<int> cycle = allv;
            cycle.pb(v);
            vector<int> cycle_vals(cycle.size());
            assert((int) cycle.size() > 1);
            int cycle_xor = 0;
            do {
                cycle_xor = 0;
                for (int i = 0; i < cycle.size() - 1; i++) {
                    cycle_vals[i] = rnd.next(0, maxc);
                    cycle_xor ^= cycle_vals[i];
                }
                cycle_vals.back() = cycle_xor;
                cycle_xor = 0;
            } while (cycle_xor != 0);
            Graph G;
            for (int i = 0; i < cycle.size(); i++) {
                G.AddEdge(cycle[i], cycle[(i + 1) % cycle.size()], cycle_vals[i]);
            }
            vector<int> path_vals(path.size() - 1);
            int path_xor = 0;
            do {
                path_xor = 0;
                for (int i = 0; i < path_vals.size(); i++) {
                    path_vals[i] = rnd.next(0, maxc);
                    path_xor ^= path_vals[i];
                }
            } while (path_xor == 0);
            for (int i = 0; i < path.size() - 1; i++) {
                G.AddEdge(path[i], path[i + 1], path_vals[i]);
            }
            G.Print();
            cout << a + 1 << " " << b + 1 << "\n";
        } else if (mode == 4) {
            vector<int> allv(n);
            for (int i = 0; i < n; i++) allv[i] = i;
            shuffle(all(allv));
            int path_size = rnd.next(3, n / 2);
            vector<int> path;
            for (int i = 0; i < path_size; i++) {
                path.pb(allv.back());
                allv.pop_back();
            }
            int a = path[0];
            int b = path.back();
            int v_id = rnd.next(1, (int) path.size() - 3);

            vector<int> cycle = allv;
            cycle.pb(path[v_id]);
            cycle.pb(path[v_id + 1]);

            vector<int> cycle_vals(cycle.size());
            assert((int) cycle.size() > 1);
            int cycle_xor = 0;
            do {
                cycle_xor = 0;
                for (int i = 0; i < cycle.size() - 1; i++) {
                    cycle_vals[i] = rnd.next(0, maxc);
                    cycle_xor ^= cycle_vals[i];
                }
                cycle_vals.back() = cycle_xor;
                cycle_xor = 0;
            } while (cycle_xor != 0);
            Graph G;
            for (int i = 0; i < cycle.size(); i++) {
                G.AddEdge(cycle[i], cycle[(i + 1) % cycle.size()], cycle_vals[i]);
            }
            vector<int> path_vals(path.size() - 1);
            int path_xor = 0;
            do {
                path_xor = 0;
                for (int i = 0; i < path_vals.size(); i++) {
                    if (i == v_id) {
                        path_xor ^= cycle_vals[cycle_vals.size() - 2];
                        continue;
                    }
                    path_vals[i] = rnd.next(0, maxc);
                    path_xor ^= path_vals[i];
                }
            } while (path_xor == 0);
            for (int i = 0; i < path.size() - 1; i++) {
                if (i == v_id) continue;
                G.AddEdge(path[i], path[i + 1], path_vals[i]);
            }
            G.Print();
            cout << a + 1 << " " << b + 1 << "\n";
        } else if (mode == 5) {
            n--;
            vector<int> allv(n);
            for (int v = 0; v < n; v++) allv[v] = v;
            shuffle(all(allv));

            int path_size = rnd.next(6, n / 2);

            vector<int> path;

            for (int i = 0; i < path_size; i++) {
                path.pb(allv.back());
                allv.pop_back();
            }

            int a = path[0], b = path.back();

            Graph G;

            int path_xor = 0;
            vector<int> path_vals;
            for (int i = 0; i + 1 < path_size; i++) {
                int w = rnd.next(0, maxc);
                if (i == path_size - 2) {
                    w = path_xor;
                }
                path_xor ^= w;

                G.AddEdge(path[i], path[i + 1], w);
                path_vals.pb(w);
            }

            int v_id = rnd.next(1, path_size / 2);
            int v = path[v_id];
            assert(v_id + 1 <= path_size - 2);
            int u_id = rnd.next(v_id + 1, path_size - 2);
            int u = path[u_id];

            int cycle_xor = 0;
            for (int i = 0; i + 1 < allv.size(); i++) {
                int w = rnd.next(0, maxc);
                cycle_xor ^= w;
                G.AddEdge(allv[i], allv[i + 1], w);
            }

            int w = rnd.next(0, maxc);
            G.AddEdge(v, allv[0], w);
            cycle_xor ^= w;
            G.AddEdge(allv.back(), v, cycle_xor);

            w = rnd.next(1, maxc);
            G.AddEdge(a, u, w);
            G.Print(true);

            cout << a + 1 << " " << b + 1 << "\n";

        } else if (mode == 6) {
            n--;
            vector<int> allv(n);
            for (int v = 0; v < n; v++) allv[v] = v;
            shuffle(all(allv));

            int path_size = rnd.next(n / 2 + 1, n - 5);

            vector<int> path;

            for (int i = 0; i < path_size; i++) {
                path.pb(allv.back());
                allv.pop_back();
            }

            int a = path[0], b = path.back();

            Graph G;

            int path_xor = 0;
            vector<int> path_vals;
            for (int i = 0; i + 1 < path_size; i++) {
                int w = rnd.next(0, maxc);
                if (i == path_size - 2) {
                    w = path_xor;
                }
                path_xor ^= w;

                G.AddEdge(path[i], path[i + 1], w);
                path_vals.pb(w);
            }

            vector<int> path2 = allv;
            int path2_size = path2.size();
            int path2_xor = 0;
            vector<int> path2_vals;
            for (int i = 0; i + 1 < path2_size; i++) {
                int w = rnd.next(0, maxc);
                if (i == path2_size - 2) {
                    w = path2_xor;
                }
                path2_xor ^= w;

                G.AddEdge(path2[i], path2[i + 1], w);
                path2_vals.pb(w);
            }
            int w = rnd.next(0, maxc);
            path2_xor ^= w;
            G.AddEdge(a, path2[0], w);
            G.AddEdge(path2.back(), b, path2_xor);


            int v_id = rnd.next(1, path2_size - 1);
            int v = path2[v_id];
            int cycle_xor = 0;
            for (int i = v_id; i + 1 < path2_size; i++) {
                cycle_xor ^= path2_vals[i];
            }
            cycle_xor ^= path2_xor;
            G.AddEdge(b, v, cycle_xor);

            G.Print(false);

            cout << a + 1 << " " << b + 1 << "\n";

        } else if (mode == 7) {
            n--;
            vector<int> allv(n);
            for (int v = 0; v < n; v++) allv[v] = v;
            shuffle(all(allv));

            int path_size = rnd.next(n / 2 + 1, n - 5);

            vector<int> path;

            for (int i = 0; i < path_size; i++) {
                path.pb(allv.back());
                allv.pop_back();
            }

            int a = path[0], b = path.back();

            Graph G;

            vector<int> path_vals;
            for (int i = 0; i + 1 < path_size; i++) {
                int w = rnd.next(0, maxc);
                G.AddEdge(path[i], path[i + 1], w);
                path_vals.pb(w);
            }

            vector<int> path2 = allv;
            int path2_size = path2.size();
            int path2_xor = 0;
            vector<int> path2_vals;
            for (int i = 0; i + 1 < path2_size; i++) {
                int w = rnd.next(0, maxc);
                if (i == path2_size - 2) {
                    w = path2_xor;
                }
                path2_xor ^= w;

                G.AddEdge(path2[i], path2[i + 1], w);
                path2_vals.pb(w);
            }
            int w = rnd.next(0, maxc);
            path2_xor ^= w;
            G.AddEdge(a, path2[0], w);
            G.AddEdge(path2.back(), b, path2_xor);


            int v_id = rnd.next(0, path2_size - 2);
            int v = path2[v_id];
            int u_id = rnd.next(v_id + 1, path2_size - 1);
            int u = path2[u_id];

            int cycle_xor = 0;
            for (int i = v_id; i < u_id; i++) {
                cycle_xor ^= path2_vals[i];
            }

            G.AddEdge(u, v, cycle_xor);

            G.Print(false);

            cout << a + 1 << " " << b + 1 << "\n";
        }
    }
    return 0;
}
