#include <iostream>
#include <vector>
#include <map>
#include <set>
#include <algorithm>

using namespace std;

#define fi first
#define se second
#define all(x) (x).begin(), (x).end()
#define pb emplace_back
#define ll long long

void dfs(int v, vector<vector<pair<int, int>>> &g, vector<bool> &usd) {
    usd[v] = true;
    for (auto &to: g[v]) {
        if (!usd[to.fi]) {
            dfs(to.fi, g, usd);
        }
    }
}

void dfs1(int v, vector<vector<pair<int, int>>> &g, vector<bool> &usd, vector<int> &cost) {
    usd[v] = true;
    for (auto &to: g[v]) {
        if (!usd[to.fi]) {
            cost[to.fi] = (cost[v] ^ to.se);

            dfs1(to.fi, g, usd, cost);
        }
    }
}

void dfs2(int v, vector<vector<int>> &g, vector<bool> &usd, vector<int> &a) {
    usd[v] = 1;
    for (auto &to: g[v]) {
        if (!usd[to]) {
            dfs2(to, g, usd, a);
        }
    }
    a.pb(v);
}


void dfs4(int v, vector<vector<pair<int, int>>> &g, vector<bool> &usd, vector<int> &cost, vector<bool> &cycle_before, vector<bool> &cycle_after) {
    usd[v] = true;
    for (auto &to: g[v]) {
        if (!cycle_before[v] && !cycle_after[to.fi]) continue;
        if (!usd[to.fi]) {
            cost[to.fi] = (cost[v] ^ to.se);

            dfs4(to.fi, g, usd, cost, cycle_before, cycle_after);
        }
    }
}

void scc(int n, vector<vector<int>> &g, vector<bool> &cycle) {
    vector<vector<int>> gr(n);
    for (int v = 0; v < n; v++) {
        for (auto &to: g[v]) {
            gr[to].pb(v);
        }
    }
    vector<int> ts;
    vector<bool> usd(n);
    for (int v = 0; v < n; v++) {
        if (!usd[v]) {
            dfs2(v, g, usd, ts);
        }
    }
    reverse(all(ts));
    usd.assign(n, false);
    for (auto &v: ts) {
        if (!usd[v]) {
            vector<int> comp;
            dfs2(v, gr, usd, comp);
            if (comp.size() > 1) {
                for (auto &u: comp) {
                    cycle[u] = true;
                }
            } else {
                bool loop = false;
                for (auto &to: g[v]) {
                    if (to == v) {
                        loop = true;
                    }
                }
                if (loop) {
                    cycle[v] = true;
                }
            }
        }
    }
}

void dfs3(int v, vector<vector<pair<int, int>>> &g, vector<bool> &processed, vector<bool> &cycle) {
    if (processed[v]) {
        return;
    }
    processed[v] = true;
    if (cycle[v]) {
        return;
    }
    for (auto &e: g[v]) {
        dfs3(e.first, g, processed, cycle);
        if (cycle[e.first]) {
            cycle[v] = true;
            return;
        }
    }
}

void dfs_dag(int v, vector<bool> &cycle_before, vector<bool> &cycle_after, vector<vector<pair<int, int>>> &gr, vector<bool> &usd, vector<int> &parity,
             vector<int> &sum) {
    if (usd[v]) {
        return;
    }
    usd[v] = true;
    for (auto &e: gr[v]) {
        dfs_dag(e.first, cycle_before, cycle_after, gr, usd, parity, sum);
        parity[v] ^= parity[e.first];
        sum[v] ^= sum[e.first];
        sum[v] ^= (e.second * parity[e.first]);
    }
}

void solve_dag(int s, int t, vector<bool> &cycle_before, vector<bool> &cycle_after, vector<vector<pair<int, int>>> &gr, int &ans) {
    int n = gr.size();
    vector<bool> usd(n);
    vector<int> parity(n);
    parity[s] = 1;
    vector<int> sum(n);
    for (int v = 0; v < n; v++) {
        dfs_dag(v, cycle_before, cycle_after, gr, usd, parity, sum);
    }
    ans = sum[t];
}

void solve_cycles(int s, int t, vector<bool> &cycle_before, vector<bool> &cycle_after, vector<vector<pair<int, int>>> &g, int &ans, vector<bool> &fromS,
                  vector<bool> &fromT) {
    if (!cycle_after[s] || !cycle_before[t]) {
        return;
    }

    int n = g.size();
    vector<bool> usd(n);

    vector<int> cost(n);
    dfs4(s, g, usd, cost, cycle_before, cycle_after);

    for (int v = 0; v < n; v++) {
        for (auto &e: g[v]) {
            int u = e.fi;
            if (cycle_before[v] || cycle_after[u]) {
                int w = e.se;
                if (fromS[v] && fromT[u]) {
                    if (w != (cost[v] ^ cost[u])) {
                        ans = -1;
                        return;
                    }
                }
            }
        }
    }
    // if (cost[t] != 0) {
    //     ans = -1;
    // }
}

void solve() {
    int n, m;
    cin >> n >> m;
    vector<vector<pair<int, int>>> g(n);
    vector<vector<pair<int, int>>> gr(n);
    while (m--) {
        int v, u, w;
        cin >> v >> u >> w;
        v--;
        u--;
        g[v].pb(u, w);
        gr[u].pb(v, w);
    }
    int s, t;
    cin >> s >> t;
    s--;
    t--;
    vector<bool> fromS(n), fromT(n);
    dfs(s, g, fromS);
    dfs(t, gr, fromT);
    if (!fromS[t]) {
        cout << 0 << "\n";
        return;
    }
    vector<vector<int>> g1(n);
    for (int v = 0; v < n; v++) {
        for (auto &e: g[v]) {
            g1[v].pb(e.first);
        }
    }
    vector<bool> cycle(n);
    scc(n, g1, cycle);
    vector<bool> processed_before(n);
    vector<bool> processed_after(n);
    vector<bool> cycle_before(n);
    vector<bool> cycle_after(n);

    for (int i = 0; i < n; i++) {
        if (!fromS[i] || !fromT[i]) {
            cycle[i] = false;
        }
        if (cycle[i]) {
            cycle_after[i] = cycle_before[i] = true;
        }
    }
    for (int v = 0; v < n; v++) {
        if (!processed_after[v]) {
            dfs3(v, g, processed_after, cycle_after);
        }
        if (!processed_before[v]) {
            dfs3(v, gr, processed_before, cycle_before);
        }
    }

    for (int i = 0; i < n; i++) {
        if (cycle_before[i] || cycle_after[i]) {
            cycle[i] = true;
        }
    }
    
    int ans = 0;
    solve_dag(s, t, cycle_before, cycle_after, gr, ans);
    solve_cycles(s, t, cycle_before, cycle_after, g, ans, fromS, fromT);
    cout << ans << "\n";
}

int main() {
    ios_base::sync_with_stdio(0);
    cin.tie(0);
    cout.tie(0);
    int t;
    cin >> t;
    while (t--) {
        solve();
    }
    return 0;
}
