//  Nikita Golikov, 2023

#include <bits/stdc++.h>

using namespace std;

using uint = unsigned int;
using ll = long long;
using ull = unsigned long long;

#ifdef GOLIKOV
    #define debug(x) cerr << (#x) << ":\t" << (x) << endl
#else
    #define debug(x) 238;
#endif

template <class A, class B>
bool smin(A& x, B&& y) {
  if (y < x) {
    x = y;
    return true;
  }
  return false;
}

template <class A, class B>
bool smax(A& x, B&& y) {
  if (x < y) {
    x = y;
    return true;
  }
  return false;
}

void solveTest() {
  int n, m;
  cin >> n >> m;
  vector<vector<pair<int, int>>> graph(n), igraph(n);
  for (int i = 0; i < m; ++i) {
    int u, v, w;
    cin >> u >> v >> w;
    --u, --v;
    graph[u].emplace_back(v, w);
    igraph[v].emplace_back(u, w);
  }
  int s, t;
  cin >> s >> t;
  --s, --t;
  vector val(2, vector(2, vector<int>(n, -1)));

  auto calcVal = [&](int id, int start, auto& g) {
    queue<pair<int, int>> q;
    val[id][0][start] = 0;
    q.emplace(0, start);
    while (!q.empty()) {
      auto[index, v] = q.front(); q.pop();
      for (auto[u, w] : g[v]) {
        int nval = val[id][index][v] ^ w;
        if (nval == val[id][0][u] || nval == val[id][1][u] || val[id][1][u] >= 0) {
          continue;
        }
        int p = val[id][0][u] >= 0;
        val[id][p][u] = nval;
        q.emplace(p, u);
      }
    }
  };

  calcVal(0, s, graph);
  calcVal(1, t, igraph);

  auto checkGood = [&](int v) {
    return val[0][0][v] >= 0 && val[1][0][v] >= 0;
  };
  vector<bool> used(n);
  vector<int> ts;

  auto dfs1 = [&](auto&& self, int v) -> void {
    used[v] = true;
    for (auto[u, w] : graph[v]) {
      if (!used[u] && checkGood(u)) {
        self(self, u);
      }
    }
    ts.push_back(v);
  };

  for (int v = 0; v < n; ++v) {
    if (checkGood(v) && !used[v]) {
      dfs1(dfs1, v);
    }
  }
  reverse(ts.begin(), ts.end());

  vector<int> id(n, -1);
  int curid = 0;

  auto dfs2 = [&](auto&& self, int v) -> void {
    id[v] = curid;
    for (auto[u, w] : igraph[v]) {
      if (checkGood(u) && id[u] < 0) {
        self(self, u);
      }
    }
  };
  for (int v : ts) {
    if (checkGood(v) && id[v] < 0) {
      dfs2(dfs2, v);
      curid++;
    }
  }

  vector<bool> haveEdge(curid);
  for (int v = 0; v < n; ++v) {
    if (checkGood(v)) {
      for (auto[u, w] : graph[v]) {
        if (checkGood(u) && id[u] == id[v]) {
          haveEdge[id[v]] = true;
        }
      }
    }
  }

  for (int v = 0; v < n; ++v) {
    if (checkGood(v) && haveEdge[id[v]]) {
      if (val[0][1][v] >= 0 || val[1][1][v] >= 0 || val[0][0][v] != val[1][0][v]) {
        cout << -1 << '\n';
        return;
      }
    }
  }

  if (!checkGood(s) || haveEdge[id[s]]) {
    cout << 0 << '\n';
    return;
  }

  vector<int> ans(n);
  vector<int> cnt(n);

  for (int i = int(ts.size()) - 1; i >= 0; --i) {
    int v = ts[i];
    if (!checkGood(v) || haveEdge[id[v]]) {
      continue;
    }
    if (v == t) {
      ans[v] = 0;
      cnt[v] = 1;
    } else {
      for (auto[u, w] : graph[v]) {
        if (checkGood(u) && !haveEdge[id[u]]) {
          ans[v] ^= ans[u];
          cnt[v] ^= cnt[u];
          if (cnt[u]) {
            ans[v] ^= w;
          }
        }
      }
    }
  }

  cout << ans[s] << '\n';
}

int main() {
#ifdef GOLIKOV
  assert(freopen("in", "rt", stdin));
  auto _clock_start = chrono::high_resolution_clock::now();
#endif
  ios::sync_with_stdio(false);
  cin.tie(nullptr);

  int tests_;
  cin >> tests_;
  for (int tt_ = 1; tt_ <= tests_; ++tt_) {
    solveTest();
  }

#ifdef GOLIKOV
  cerr << "Executed in " << chrono::duration_cast<chrono::milliseconds>(
      chrono::high_resolution_clock::now()
          - _clock_start).count() << "ms." << endl;
#endif
  return 0;
}