#include <bits/stdc++.h>
using namespace std;

typedef long long ll;
typedef long double ldb;

#define  all(v) (v).begin(),(v).end()
#define sz(v) ((int)(v).size())

vector<vector<pair<int, int>>> readGraph(int n, int m) {
    vector<vector<pair<int, int>>> g(n);
    for (int i = 0; i < m; ++i) {
        int a, b, c;
        cin >> a >> b >> c;
        a--;
        b--;
        g[a].push_back({b, c});
    }
    return g;
}

vector<vector<pair<int, int>>> reverseGraph(const vector<vector<pair<int, int>>> &g) {
    int n = g.size();
    vector<vector<pair<int, int>>> g2(n);
    for (int i = 0; i < n; ++i) {
        for (auto pp : g[i]) {
            g2[pp.first].push_back({i, pp.second});
        }
    }
    return g2;
}

vector<int> reachable(const vector<vector<pair<int, int>>> &g, int start) {
    int n = g.size();
    vector<int> used(n);
    used[start] = 1;
    deque<int> q;
    q.push_back(start);
    while (!q.empty()) {
        int v = q.front();
        q.pop_front();
        for (auto pp : g[v]) {
            int u = pp.first;
            if (!used[u]) {
                used[u] = 1;
                q.push_back(u);
            }
        }
    }
    return used;
}

vector<vector<pair<int, int>>> induce(const vector<vector<pair<int, int>>> &g, const vector<int> &need) {
    int n = g.size();
    vector<vector<pair<int, int>>> g2(n);
    for (int i = 0; i < n; ++i) {
        if (need[i]) {
            for (auto pp : g[i]) {
                if (need[pp.first]) {
                    g2[i].push_back(pp);
                }
            }
        }
    }
    return g2;
}

vector<vector<int>> getXors(const vector<vector<pair<int, int>>> &g, int start) {
    int n = g.size();
    vector<vector<int>> ans(n);
    ans[start] = {0};
    set<pair<int, int>> q;
    q.insert({0, start});
    vector<int> used(n);
    vector<int> deg(n);
    for (int i = 0; i < n; ++i) {
        for (auto pp : g[i]) {
            deg[pp.first]++;
        }
    }
    while (!q.empty()) {
        int v = q.begin()->second;
        q.erase(q.begin());
        used[v] = 1;
        for (auto pp : g[v]) {
            for (int val : ans[v]) {
                if (ans[pp.first].size() < 2 && find(all(ans[pp.first]), val ^ pp.second) == ans[pp.first].end()) {
                    ans[pp.first].push_back(val ^ pp.second);
                }
            }
            q.erase({deg[pp.first], pp.first});
            deg[pp.first]--;
            if (!used[pp.first]) {
                q.insert({deg[pp.first], pp.first});
            }
        }
    }
    return ans;
}

vector<int> getBad(const vector<vector<pair<int, int>>> &g, const vector<vector<int>> &xors) {
    int n = g.size();
    deque<int> q;
    vector<int> used(n);
    for (int i = 0; i < n; ++i) {
        if (xors[i].size() == 2) {
            used[i] = 1;
            q.push_back(i);
        }
    }
    while (!q.empty()) {
        int v = q.front();
        q.pop_front();
        for (auto pp : g[v]) {
            int u = pp.first;
            if (!used[u]) {
                used[u] = 1;
                q.push_back(u);
            }
        }
    }
    return used;
}

void solve() {
#define NO cout << "-1\n";return
    int n, m;
    cin >> n >> m;
    int a, b;
    vector<vector<pair<int, int>>> g;
    {
        // remove all not visited vertices
        g = readGraph(n, m);
        auto gr = reverseGraph(g);
        cin >> a >> b;
        a--;
        b--;
        auto reach1 = reachable(g, a);
        auto reach2 = reachable(gr, b);
        for (int i = 0; i < n; ++i) {
            reach1[i] &= reach2[i];
        }
        g = induce(g, reach1);
    }
    if (0) {
        for (int i = 0; i < n; ++i) {
            for (auto pp : g[i]) {
                cout << i << " " << pp.first << " " << pp.second << endl;
            }
        }
        return;
    }
    auto g2 = g;
    vector<array<int, 3>> backEdges;
    {
        // separate back edges from graph edges
        vector<vector<pair<int, int>>> g2(n);  
        vector<int> used(n);
        auto dfs = [&](auto dfs, int v) -> void {
            used[v] = 1;
            for (auto pp : g[v]) {
                int u = pp.first;
                if (!used[u]) {
                    g2[v].push_back(pp);
                    dfs(dfs, u);
                } else if (used[u] == 2) {
                    g2[v].push_back(pp);
                } else {
                    backEdges.push_back({v, u, pp.second});
                }
            }
            used[v] = 2;
        };
        dfs(dfs, a);
        g = g2;
    }
    auto gr = reverseGraph(g2);
    auto x1 = getXors(g2, a);
    vector<int> bad = getBad(g2, x1);
    auto x2 = getXors(gr, b);
    vector<int> bad2 = getBad(gr, x2);
    if (bad[a] || bad2[b]) {
        NO;
    }
    for (auto edge : backEdges) {
        int from = edge[0];
        int to = edge[1];
        int w = edge[2];
        if (x1[from].size() > 1 || x2[to].size() > 1) {
            NO;
        }
        bool bad = false;
        auto cassert = [&](bool b) {
            if (!b) {
                bad = 1;
            }
        };
        cassert(x1[from].size() == 1);
        cassert(x1[to].size() == 1);
        cassert(x2[from].size() == 1);
        cassert(x2[to].size() == 1);
        if (bad) {
            NO;
        }
        if ((x1[from][0] ^ x2[to][0] ^ w) != 0) {
            NO;
        }
        if (x1[from][0] != x2[from][0]) {
            NO;
        }
    }
    vector<int> ans(n);
    vector<int> cnt(n);
    cnt[a] = 1;
    deque<int> q;
    q.push_back(a);
    vector<int> deg(n);
    for (int i = 0; i < n; ++i) {
        for (auto pp : g[i]) {
            deg[pp.first]++;
        }
    }
    while (!q.empty()) {
        int v = q.front();
        q.pop_front();
        for (auto pp : g[v]) {
            cnt[pp.first] ^= cnt[v];
            ans[pp.first] ^= ans[v] ^ (cnt[v] * pp.second);
            deg[pp.first]--;
            if (deg[pp.first] == 0) {
                q.push_back(pp.first);
            }
        }
    }
    cout << ans[b] << endl;
}

signed main() {
    ios_base::sync_with_stdio(0);
    cin.tie(0);
    cout.tie(0);
    int t = 1;
    cin >> t;
    while (t--) {
        solve();
    }
}