#include "testlib.h"
#include <bits/stdc++.h>

#define x first
#define y second

using namespace std;

map<string, string> params;

long long get(string name, long long def = 0) {
    if (params.count(name)) {
        return stoll(params[name]);
    } else {
        return def;
    }
}

char get_ch(string name, char def = 'a') {
    if (params.count(name)) {
        return params[name][0];
    } else {
        return def;
    }
}

vector<int> parts(int n, int k, int dist = 1) {
    vector<int> idx(k + 1);
    for (int i = 1; i < k; i++) {
        idx[i] = rnd.next(0, n - dist * k);
    }
    idx[k] = n;
    sort(idx.begin(), idx.end());
    for (int i = 0; i < k; i++) {
        idx[i] += dist * i;
    }
    return idx;
}

const int maxx = 1'000'000;
const int maxn = 500'000;

string t;
vector<string> s;
vector<pair<int, int>> q;

int n;
int m;
int tlen;
int slen;


void add_qlen(int max_len) {
    int qlen = rnd.next(1, max_len);
    int l = rnd.next(0, tlen - qlen);
    q.emplace_back(l, l + qlen - 1);
}

void gen_queries() {
    string qmode = params["qmode"];
    if (qmode == "random") {
        for (int i = 0; i < m; i++) {
            int l = rnd.next(tlen);
            int r = rnd.next(tlen);
            if (l > r) {
                swap(l, r);
            }  
            q.emplace_back(l, r);
        }
    } else if (qmode == "ends") {
        for (int i = 0; i < (int)sqrt(m); i++) {
            for (int j = 0; j < (int)sqrt(m); j++) {
                int l = min(i, tlen - 1);
                int r = max(tlen - 1 - j, 0);
                if (r < l) {
                    swap(l, r);
                }
                q.emplace_back(l, r);
            }
        }
    } else if (qmode == "small_large") {
        int small = get("small_queries", m / 2);
        int small_max_len = get("small_max_len", sqrt(tlen));

        // cout << "SSS " << small << '\n';

        for (int i = 0; i < small; i++) {
            add_qlen(small_max_len);
        }

        for (int i = 0; i < m - small; i++) {
            add_qlen(tlen);
        }
    }
}

void generic_gen() {
    int alph = get("alph", 26);

    string tmode = params["tmode"];
    t.resize(tlen);

    if (tmode == "random") {
        for (int i = 0; i < tlen; i++) {
            t[i] = rnd.next(alph) + 'a';
        }
    } else if (tmode == "thue_morse") {
        for (int i = 0; i < tlen; i++) {
            t[i] = 'a' + __builtin_popcount(i) % 2;
        }
    } else if (tmode == "palindrome") {
        int part_size = get("part_size", 1);
        int i = 0;
        while (i < tlen) {
            int ts = i;
            int ps = rnd.next(1, part_size);
            for (int j = 0; j < ps && i < tlen; j++) {
                t[i++] = rnd.next(alph) + 'a';
            }
            for (int j = 0; j < ts && i < tlen; j++) {
                t[i++] = t[ts - j - 1];
            }
        }
    } else if (tmode == "combination") {
        vector<string> ss;
        for (int i = 0; i < 2; i++) {
            int l = rnd.next(5, 10);
            ss.emplace_back(l, 'a');
            for (int j = 0; j < l; j++) {
                ss.back()[j] = rnd.next(alph) + 'a';
            }
        }

        t.resize(0);
        while (t.size() < tlen) {
            t += ss[rnd.next(2)];
        }
        t.resize(tlen);
    }
    

    string smode = "random";
    if (params.count("smode")) {
        smode = params["smode"];
    }

    vector<int> lens;
    bool gens = true;
    if (smode == "random") {
        lens = rnd.partition(n, slen);
    } else if (smode == "same") {
        lens.resize(n, slen / n);
    } else if (smode == "all") {
        for (int i = 1; i <= slen && i <= n; i++) {
            lens.emplace_back(i);
            slen -= i;
        }
    } else if (smode == "small_large") {
        int large_len = get("large_len", slen / 2);
        int large_cnt = get("large_cnt", sqrt(large_len));

        lens = rnd.partition(large_cnt, large_len);

        auto small_lens = rnd.partition(n - large_cnt, slen - large_len);
        for (auto i : small_lens) {
            lens.emplace_back(i);
        }
    } else if (smode == "thue_morse") {
        gens = false;
        int l = get("thue_len", slen);

        int lst = 1;
        while (l >= lst && lst <= tlen) {
            int x = rnd.next(0, 1);
            s.emplace_back(lst, 'a');
            for (int i = 0; i < lst; i++) {
                s.back()[i] = 'a' + (__builtin_popcount(i) + x) % 2;
            }
            l -= lst;
            slen -= lst;
            lst *= 2;
            n--;
        }

        if (n > slen) {
            n = sqrt(slen);
        }
        if (n > 0) {
            lens = rnd.partition(n, slen);
        }
    }

    string sgen = "random";
    if (params.count("sgen")) {
        sgen = params["sgen"];
    }

    if (sgen == "random") {
        for (auto i : lens) {
            s.emplace_back(i, 'a');
            for (int j = 0; j < i; j++) {
                s.back()[j] = rnd.next(alph) + 'a';
            }
        }
    } else if (sgen == "substr") {
        for (auto i : lens) {
            s.emplace_back(i, 'a');
            if (i > tlen) {
                for (int j = 0; j < i; j++) {
                    s.back()[j] = rnd.next(alph) + 'a';
                }
            } else {
                int bg = rnd.next(0, tlen - i);
                for (int j = 0; j < i; j++) {
                    s.back()[j] = t[j + bg];
                }
            }
        }
    }

    gen_queries();
}

void gen_aaab() {
    t.resize(tlen, 'a');

    t[0] = 'b';
    t[tlen - 1] = 'b';

    for (int i = 1; i * 3 + 2 < slen && s.size() + 3 < n; i++) {
        s.emplace_back(i, 'a');
        s.emplace_back(i + 1, 'a');
        s.back().back() = 'b';
        s.emplace_back(i + 1, 'a');
        s.back()[0] = 'b';
        slen -= i * 3 + 2;
    }

    for (int i = 0; i * 3 + 3 <= m; i++) {
        int qlen = rnd.next(tlen);
        // cout << qlen << '\n';
        q.emplace_back(0, qlen);
        qlen = rnd.next(tlen);
        // cout << qlen << '\n';
        q.emplace_back(tlen - qlen - 1, tlen - 1);
        add_qlen(tlen);
    }
}

void gen_aabb() {
    int tt = 0;
    for (; tt + 4 < tlen && tt * 3 + 5 < slen && s.size() + 3 <= n; tt += 3) {
        s.emplace_back(tt + 1, 'a');
        s.emplace_back(tt + 2, 'a');
        s.back()[tt] = 'b';
        s.back()[tt + 1] = 'b';
        s.emplace_back(tt + 2, 'a');
        s.back()[0] = 'b';
        s.back()[1] = 'b';
        slen -= tt * 3 + 5;
    }
    t.resize(tt + 4, 'a');
    t[tt + 2] = 'b';
    t[tt + 3] = 'b';
    t[0] = 'b';
    t[1] = 'b';
    tt += 4;
    while (t.size() + tt < tlen) {
        for (int i = 0; i < tt; i++) {
            t += t[i];
        }
    }

    tlen = t.size();

    for (int i = 0; i < m; i++) {
        int mod = rnd.next(0, 6);
        if (mod == 6) {
            add_qlen(tlen);
        } else if (mod <= 2) {
            int r = tt - mod - 1;
            int l = rnd.next(0, r);
            int add = rnd.next(tlen / tt);
            q.emplace_back(l + add * tt, r + add * tt);
        } else {
            mod -= 3;
            int l = mod;
            int r = rnd.next(l, tt - 1);
            int add = rnd.next(tlen / tt);
            q.emplace_back(l + add * tt, r + add * tt);
        }
    }
}

void gen_abab() {
    t.resize(tlen, 'a');
    for (int i = 1; i < tlen; i += 2) {
        t[i] = 'b';
    }

    int st = get("start", min(tlen / 2, slen));

    for (int j = st; j <= tlen && n > s.size() && j <= slen; j++) {
        slen -= j;
        s.emplace_back(j, 'a');
        for (int i = 1; i < j; i += 2) {
            s.back()[i] = 'b';
        }
        if (slen >= j && n > s.size()) {
            slen -= j;
            s.emplace_back(j, 'a');
            for (int i = 0; i < j; i += 2) {
                s.back()[i] = 'b';
            }
        }
    }

    gen_queries();
}

void post_process() {
    set<string> sset;
    for (auto i : s) {
        sset.insert(i);
    }
    s.clear();
    for (auto i : sset) {
        s.emplace_back(i);
    }

    if (!params.count("no_shuffle")) {
        shuffle(s.begin(), s.end());
        shuffle(q.begin(), q.end());
    }
}


int main(int argc, char* argv[]) {
    registerGen(argc, argv, 1);
    for (int i = 1; i < argc; i++) {
        string s = string(argv[i]);
        if (s.find("=") != string::npos) {
            auto pos = s.find("=");
            params[s.substr(0, pos)] = s.substr(pos + 1);
        } else {
            params[s] = "";
        }
    }

    n = get("n", maxn);
    m = get("m", maxn);
    tlen = get("t", maxx);
    slen = get("s", maxx);

    string mode = params["mode"];

    if (mode == "") {
        generic_gen();
    } else if (mode == "aaab") {
        gen_aaab();
    } else if (mode == "aabb") {
        gen_aabb();
    } else if (mode == "abab") {
        gen_abab();
    }

    post_process();

    cout << s.size() << ' ' << q.size() << '\n';
    cout << t << '\n';
    for (auto i : s) {
        cout << i << '\n';
    }
    for (auto i : q) {
        cout << i.x + 1 << ' ' << i.y + 1 << '\n';
    }
}
