//  Nikita Golikov, 2023

#include <bits/stdc++.h>

using namespace std;

using uint = unsigned int;
using ll = long long;
using ull = unsigned long long;

#ifdef GOLIKOV
    #define debug(x) cerr << (#x) << ":\t" << (x) << endl
#else
    #define debug(x) 238;
#endif

template <class A, class B>
bool smin(A& x, B&& y) {
  if (y < x) {
    x = y;
    return true;
  }
  return false;
}

template <class A, class B>
bool smax(A& x, B&& y) {
  if (x < y) {
    x = y;
    return true;
  }
  return false;
}

template <int ALPHA, char START>
struct aho {
  struct node {
    int suff = -1;
    bool term = false;
    int nterm = -1;
    int length{};
    int nxt[ALPHA]{};
    int go[ALPHA]{};
  };
  vector<node> trie;
  vector<int> q;
  bool prepared = false;

  aho() {
    trie.emplace_back();
  }

  template <class T>
  int add_string(T const& s) {
    int cur = 0;
    for (auto ch : s) {
      if (!trie[cur].nxt[ch - START]) {
        trie[cur].nxt[ch - START] = (int) trie.size();
        trie.emplace_back();
        trie.back().length = trie[cur].length + 1;
      }
      cur = trie[cur].nxt[ch - START];
    }
    trie[cur].term = true;
    return cur;
  }

  void prepare_links() {
    prepared = true;
    q.push_back(0);
    for (int ptr = 0; ptr < (int) q.size(); ++ptr) {
      auto v = q[ptr];
      for (auto ch = 0; ch < ALPHA; ++ch) {
        auto u = trie[v].nxt[ch];
        int sf = trie[v].suff;
        if (!u) {
          trie[v].go[ch] = sf < 0 ? 0 : trie[sf].go[ch];
          continue;
        }
        trie[v].go[ch] = u;
        trie[u].suff = sf < 0 ? 0 : trie[sf].go[ch];
        if (trie[u].term) {
          trie[u].nterm = u;
        } else {
          trie[u].nterm = trie[trie[u].suff].nterm;
        }
        q.push_back(u);
      }
    }
  }

  ~aho() {
    if (!prepared) {
      cerr << "PREPARE LINKS!!!\n";
    }
  }
};

int main() {
#ifdef GOLIKOV
  assert(freopen("in", "rt", stdin));
  auto _clock_start = chrono::high_resolution_clock::now();
#endif
  ios::sync_with_stdio(false);
  cin.tie(nullptr);

  int m, q;
  cin >> m >> q;
  string t; cin >> t;
  int n = int(t.size());
  aho<26, 'a'> dict{};
  for (int i = 0; i < m; ++i) {
    string s; cin >> s; dict.add_string(s);
  }
  dict.prepare_links();
  vector<vector<pair<int, int>>> qsR(n);
  for (int i = 0; i < q; ++i) {
    int l, r;
    cin >> l >> r;
    --l, --r;
    qsR[r].emplace_back(l, i);
  }
  vector<ll> ans(q);

  int const B = 500;
  vector<ll> addBlock((n + B - 1) / B);
  vector<ll> add(n);

  int curv = 0;
  for (int r = 0; r < n; ++r) {
    curv = dict.trie[curv].go[t[r] - 'a'];
    int v = curv;
    while (v > 0) {
      v = dict.trie[v].nterm;
      if (v < 0) {
        break;
      }
      int L = r - dict.trie[v].length + 1;
      ++add[L];
      ++addBlock[L / B];
      v = dict.trie[v].suff;
    }
    for (auto[l, i] : qsR[r]) {
      ll res = 0;
      while (l <= r && l % B) {
        res += add[l++];
      }
      if (l <= r) {
        int lb = l / B;
        int rb = r / B;
        for (int j = lb; j <= rb; ++j) {
          res += addBlock[j];
        }  
      }
      ans[i] = res;
    }
  }

  for (auto x : ans) {
    cout << x << ' ';
  }
  cout << '\n';

#ifdef GOLIKOV
  cerr << "Executed in " << chrono::duration_cast<chrono::milliseconds>(
      chrono::high_resolution_clock::now()
          - _clock_start).count() << "ms." << endl;
#endif
  return 0;
}
