#include <bits/stdc++.h>
using namespace std;

using ll = long long;
using ld = long double;

#define all(a) begin(a), end(a)
#define len(a) int((a).size())

int main() {
    cin.tie(nullptr)->sync_with_stdio(false);

    struct node {
        array<node*, 26> nxt = {};
        node* par = nullptr, *link = nullptr, *slink = nullptr;
        int par_symbol = -1, length = 0, depth = 0, string_id = -1;
    };

    auto add_string = [&](const string &s, int id, node* v) {
        for (char c : s) {
            if (!v->nxt[c - 'a']) {
                v->nxt[c - 'a'] = new node();
                v->nxt[c - 'a']->par = v;
                v->nxt[c - 'a']->par_symbol = c - 'a';
                v->nxt[c - 'a']->length = v->length + 1;
            }
            v = v->nxt[c - 'a'];
        }
        v->string_id = id;
    };

    auto initialize_aho = [&](node* root) {
        queue<node*> que;
        que.push(root);
        while (!que.empty()) {
            node* v = que.front();
            que.pop();
            if (v != root) {
                v->link = v->par == root ? root : v->par->link->nxt[v->par_symbol];
                v->slink = v->string_id == -1 ? v->link->slink : v;
                v->depth = v->link->depth + (v->string_id != -1);
            }
            for (int i = 0; i < 26; i++) {
                if (v->nxt[i]) {
                    que.push(v->nxt[i]);
                } else {
                    v->nxt[i] = v == root ? root : v->link->nxt[i];
                }
            }
        }
    };

    int n, q;
    string s;
    cin >> n >> q >> s;

    node* root[2];
    root[0] = new node();
    root[1] = new node();
    vector<string> a(n);
    for (int i = 0; i < n; i++) {
        cin >> a[i];
        add_string(a[i], i, root[0]);
        add_string(string(a[i].rbegin(), a[i].rend()), i, root[1]);
    }
    initialize_aho(root[0]);
    initialize_aho(root[1]);

    auto find_pref_suf = [&](const string &s) {
        int n = len(s);
        vector<ll> pref(n + 1), suf(n + 1);
        node* v = root[0];
        for (int i = 0; i < n; i++) {
            v = v->nxt[s[i] - 'a'];
            pref[i + 1] = pref[i] + v->depth;
        }
        v = root[1];
        for (int i = n - 1; i >= 0; i--) {
            v = v->nxt[s[i] - 'a'];
            suf[i] = suf[i + 1] + v->depth;
        }
        return make_pair(pref, suf);
    };

    auto [s_pref, s_suf] = find_pref_suf(s);
    vector<vector<ll>> pref(n), suf(n);
    vector<vector<int>> real_string(n);
    for (int i = 0; i < n; i++) {
        tie(pref[i], suf[i]) = find_pref_suf(a[i]);

        real_string[i].resize(len(a[i]), -1);
        node* v = root[1];
        for (int j = len(a[i]) - 1, ptr = j; j >= 0; j--) {
            v = v->nxt[a[i][j] - 'a'];
            if (v->string_id == -1) continue;
            for (; ptr >= j; ptr--) {
                real_string[i][ptr] = v->string_id;
            }
        }
    }

    struct query {
        int l, r, id;
    };
    vector<query> queries(q);
    vector<ll> ans(q);
    for (int i = 0; i < q; i++) {
        cin >> queries[i].l >> queries[i].r;
        queries[i].l--, queries[i].r--;
        queries[i].id = i;
    }
    sort(all(queries), [&](const auto &a, const auto &b) { return a.r < b.r; });

    auto query = [&](const vector<ll> &pref, const vector<ll> &suf, int l, int r, bool term) {
        if (l == 0) return pref[r + 1];
        if (r == len(pref) - 2) return suf[l];
        return pref[r + 1] + suf[l] - pref.back() + term;
    };

    priority_queue<array<int, 3>> pq;
    node* v = root[0];
    for (int i = 0, j = 0; i < len(s); i++) {
        for (; j < len(queries) && queries[j].r == i; j++) {
            pq.push({queries[j].l, queries[j].r, queries[j].id});
        }
        v = v->nxt[s[i] - 'a'];
        if (!v->slink) continue;

        int start = i - v->slink->length + 1;
        for (; !pq.empty() && pq.top()[0] >= start; pq.pop()) {
            auto [l, r, id] = pq.top();
            int string_id = real_string[v->slink->string_id][l - start];
            int new_start = i - len(a[string_id]) + 1;
            ans[id] = query(pref[string_id], suf[string_id], l - new_start, r - new_start, true);
        }
    }
    for (; !pq.empty(); pq.pop()) {
        auto [l, r, id] = pq.top();
        ans[id] = query(s_pref, s_suf, l, r, false);
    }

    for (int i = 0; i < q; i++) {
        cout << ans[i] << " \n"[i == q - 1];
    }
}
