#include <bits/stdc++.h>
#include "testlib.h"

using namespace std;

const int kMaxQ = 3500;

typedef long long ll;

ll GetSegmentSum(int L, int R) {
    return (ll)(R - L + 1) * (ll)(L + R) / 2LL;
}

// checks if interactor answers have at least one solution
void VerifyInteractorCorrectness(int n, int k, const map<int, ll>& cache) {
    vector<pair<int, ll>> pref(cache.begin(), cache.end());
    pref.insert(pref.begin(), make_pair(0, 0LL));
    int kL = 0;
    int kR = 0;
    for (int i = 0; i + 1 < (int)pref.size() && kL <= k; i++) {
        int L = pref[i].first + 1;
        int R = pref[i + 1].first;
        ll sum = pref[i + 1].second - pref[i].second;
        if (sum < 0) {
            tout << -2 << endl;
            cout << -1 << endl << flush;
            ouf.quitf(_fail, "interactor fail: no solution to answers exists");
        }
        sum = GetSegmentSum(L, R) - sum;
        if (sum < 0) {
            tout << -2 << endl;
            cout << -1 << endl << flush;
            ouf.quitf(_fail, "interactor fail: no solution to answers exists");
        }
        if (sum == 0) {
            continue;
        }
        int cnt_r = min((ll)R - (ll)L + 1, sum / L);
        int cnt_l = min((ll)R - (ll)L + 1, (sum + R - 1) / R);
        if (cnt_l > k) {
            tout << -2 << endl;
            cout << -1 << endl << flush;
            ouf.quitf(_fail, "interactor fail: no solution to answers exists");
        }
        cnt_r = min(cnt_r, k);
        while (cnt_l <= cnt_r && (GetSegmentSum(L, L + cnt_l - 1) > sum || sum > GetSegmentSum(R - cnt_l + 1, R))) {
            cnt_l++;
        }
        while (cnt_l <= cnt_r && (GetSegmentSum(L, L + cnt_r - 1) > sum || sum > GetSegmentSum(R - cnt_r + 1, R))) {
            cnt_r--;
        }
        if (cnt_l > cnt_r) {
            tout << -2 << endl;
            cout << -1 << endl << flush;
            ouf.quitf(_fail, "interactor fail: no solution to answers exists");
        }
        kL += cnt_l;
        kR += cnt_r;
    }
    kR += n - pref.back().first;
    if (k < kL || k > kR) {
        tout << -2 << endl;
        cout << -1 << endl << flush;
        ouf.quitf(_fail, "interactor fail: no solution to answers exists");
    }
}

// checks if pos is the only vector that satisfy answers on queries given in cache
void Verify(int n, int k, const map<int, ll>& cache, const vector<int>& pos) {
    vector<pair<int, ll>> pref(cache.begin(), cache.end());
    pref.insert(pref.begin(), make_pair(0, 0LL));
    int cnt_pos = 0;
    int cnt_neg = 0;
    int remains = k;
    for (int i = 0; i + 1 < (int)pref.size(); i++) {
        int L = pref[i].first + 1;
        int R = pref[i + 1].first;
        ll sum = pref[i + 1].second - pref[i].second;
        sum = GetSegmentSum(L, R) - sum;
        int pl = lower_bound(pos.begin(), pos.end(), L) - pos.begin();
        int pr = lower_bound(pos.begin(), pos.end(), R + 1) - pos.begin();
        ll current_sum = 0;
        for (int j = pl; j < pr; j++) {
            current_sum += pos[j];
        }
        if (sum != current_sum) {
            tout << -1 << endl;
            cout << -1 << endl << flush;
            ouf.quitf(_wa, "the answer is incorrect");
        }
        int count = pr - pl;
        remains -= count;
        if (sum == 0 || sum == GetSegmentSum(L, R)) {
            continue;
        }
        if (GetSegmentSum(L, L + count - 2) <= sum && sum <= GetSegmentSum(R - count + 2, R)) {
            cnt_neg++;
        }
        if (GetSegmentSum(L, L + count) <= sum && sum <= GetSegmentSum(R - count, R)) {
            cnt_pos++;
        }
        if (count <= 1) {
            continue;
        }
        if (R - L + 1 - count <= 1) {
            continue;
        }
        if (pos[pr - 1] - L + 1 == count || R - pos[pl] + 1 == count) {
            continue;
        }
        if (pos[pr - 1] - L == count && pos[pr - 1] == pos[pr - 2] + 2) {
            continue;
        }
        if (R - pos[pl] == count && pos[pl + 1] == pos[pl] + 2) {
            continue;
        }
        tout << -1 << endl;
        cout << -1 << endl << flush;
        ouf.quitf(_wa, "the answer is not unique");
    }
    if (remains > 0) {
        cnt_neg++;
    }
    if (remains < n - pref.back().first) {
        cnt_pos++;
    }
    if (cnt_pos > 0 && cnt_neg > 0) {
        tout << -1 << endl;
        cout << -1 << endl << flush;
        ouf.quitf(_wa, "the answer is not unique");
    }
    cout << 1 << endl << flush;
}

class Interactor {
public:
    virtual ~Interactor() = default;

    virtual ll GetSum(int p) = 0;

    virtual int GetN() = 0;

    virtual int GetK() = 0;
};

class TSetInteractor : public Interactor {
public:
    TSetInteractor() {
        n = inf.readInt();
        k = inf.readInt();
        vector<int> pos(k);
        for (int i = 0; i < k; i++) {
            pos[i] = inf.readInt();
        }
        pref.reserve(k + 1);
        ll sum = 0;
        for (int i = 0; i < k; i++) {
            pref.emplace_back(pos[i], sum);
            sum += pos[i];
        }
        pref.emplace_back(n + 1, sum);
    }

    ll GetSum(int p) override {
        ll res = GetSegmentSum(1, p);
        res -= upper_bound(pref.begin(), pref.end(), make_pair(p + 1, -1LL))->second;
        return res;
    }

    int GetN() override {
        return n;
    }

    int GetK() override {
        return k;
    }

private:
    int n, k;
    vector<pair<int, ll>> pref;
};

pair<int, int> GetPossibleSegment(int k, int l, int r, ll s) {
    if (s == 0) {
        return make_pair(0, 0);
    }
    k = min(k, r - l + 1);
    if (s < l || s > GetSegmentSum(r - k + 1, r)) {
        return make_pair(-1, -1);
    }
    int sl = -1;
    int sr = -1;
    {
        int L = 1;
        int R = k + 1;
        while (R - L > 1) {
            int h = (L + R) / 2;
            if (GetSegmentSum(l, l + h - 1) <= s) {
                L = h;
            } else {
                R = h;
            }
        }
        sr = L;
    }
    {
        int L = 0;
        int R = k;
        while (R - L > 1) {
            int h = (L + R) / 2;
            if (GetSegmentSum(r - h + 1, r) >= s) {
                R = h;
            } else {
                L = h;
            }
        }
        sl = R;
    }
    if (sl > sr) {
        sl = sr = -1;
    }
    return make_pair(sl, sr);
}

class TAdaptiveInteractor : public Interactor {
public:
    TAdaptiveInteractor() {
        n = inf.readInt();
        k = inf.readInt();
        tot_s = inf.readLong();
        pref.insert({n, tot_s});
        pref.insert({0, 0});
        auto seg = GetPossibleSegment(k, 1, n, tot_s);
        assert(seg.second == k);
        sum_l = seg.first;
        sum_r = seg.second;
    }

    ll GetSum(int p) override {
        if (p == n) {
            return GetSegmentSum(1, p) - tot_s;
        }

        auto next_it = pref.lower_bound({p + 1, -1LL});
        int R = next_it->first;
        ll sR = next_it->second;
        int L = prev(next_it)->first;
        ll sL = prev(next_it)->second;
        ll s = sR - sL;
        if (s == 0) {
            return GetSegmentSum(1, p) - sL;
        }

        L++;
        {
            auto seg = GetPossibleSegment(k, L, R, s);
            sum_l -= seg.first;
            sum_r -= seg.second;
        }

        ll to_l = -1;
        pair<int, double> best_pot = {-1, -1};
        auto upd_pot = [&](ll go_l) {
            ll go_r = s - go_l;
            auto seg_l = GetPossibleSegment(k, L, p, go_l);
            auto seg_r = GetPossibleSegment(k, p + 1, R, go_r);
            pair<int, double> cur_pot(0, 0);
            if (seg_l.first != 0) {
                cur_pot.first += seg_l.second - seg_l.first + 1;
            }
            if (seg_r.first != 0) {
                cur_pot.first += seg_r.second - seg_r.first + 1;
            }
            cur_pot.second += ((double)go_l / L) - ((double)go_l / p);
            cur_pot.second += ((double)go_r / (p + 1)) - ((double)go_r / R);
            if (cur_pot > best_pot) {
                best_pot = cur_pot;
                to_l = go_l;
            }
        };

        for (int sm = max(1, k - sum_r); sm <= k - sum_l; sm++) {
            for (int kl = 0; kl <= sm; kl++) {
                int kr = sm - kl;
                if (kl > p - L + 1 || kr > R - p) {
                    continue;
                }
                ll s1_left = GetSegmentSum(L, L + kl - 1);
                ll s1_right = GetSegmentSum(p - kl + 1, p);
                ll s2_left = GetSegmentSum(p + 1, p + kr);
                ll s2_right = GetSegmentSum(R - kr + 1, R);
                if (s1_left + s2_left <= s && s <= s1_right + s2_right) {
                    ll seg_l = max(s1_left, s - s2_right);
                    ll seg_r = min(s1_right, s - s2_left);
                    assert(seg_l <= seg_r);
                    upd_pot(seg_l);
                    if (seg_l != seg_r) {
                        upd_pot(seg_r);
                        upd_pot((seg_l + seg_r) / 2);
                    }
                }
            }
        }

        assert(to_l != -1);
        ll cur_sum = sL + to_l;
        pref.insert({p, cur_sum});
        {
            auto seg = GetPossibleSegment(k, L, p, to_l);
            sum_l += seg.first;
            sum_r += seg.second;
        }
        {
            auto seg = GetPossibleSegment(k, p + 1, R, s - to_l);
            sum_l += seg.first;
            sum_r += seg.second;
        }
        assert(sum_l <= k && k <= sum_r);

        return GetSegmentSum(1, p) - cur_sum;
    }

    int GetN() override {
        return n;
    }

    int GetK() override {
        return k;
    }

private:
    int n, k;
    ll tot_s;
    set<pair<int, ll>> pref;
    int sum_l, sum_r;
};

unique_ptr<Interactor> ConstructInteractor(const std::string& type) {
    if (type == "set") {
        return make_unique<TSetInteractor>();
    } else if (type == "adaptive") {
        return make_unique<TAdaptiveInteractor>();
    } else {
        assert(false);
    }
}

int Simulate(unique_ptr<Interactor> interactor, int maxq = kMaxQ) {
    map<int, ll> cache;
    int q_count = 0;
    int n = interactor->GetN();
    int k = interactor->GetK();

    cout << n << " " << k << endl << flush;

    while (true) {
        string type = ouf.readToken("?|!", format("type[%d]", q_count + 1));

        if (type == "!") {
            vector<int> res(k);
            for (int i = 0; i < k; i++) {
                res[i] = ouf.readInt(1, n, format("res[%d]", i + 1));
            }
            sort(res.begin(), res.end());
            for (int i = 0; i < k - 1; i++) {
                if (res[i] == res[i + 1]) {
                    tout << -1 << endl;
                    cout << -1 << endl << flush;
                    ouf.quitf(_wa, "two equal positions in the set");
                }
            }
            VerifyInteractorCorrectness(n, k, cache);
            Verify(n, k, cache, res);
            return q_count;
        }

        ++q_count;
        if (q_count > maxq) {
            tout << -1 << endl;
            cout << -1 << endl << flush;
            ouf.quitf(_wa, "too many queries");
        }

        int p = ouf.readInt(1, n, format("query[%d]", q_count));
        if (cache.find(p) != cache.end()) {
            cout << cache[p] << endl << flush;
            continue;
        }

        ll res = interactor->GetSum(p);
        cache[p] = res;
        cout << res << endl << flush;
    }
}

int main(int argc, char* argv[]) {
    registerInteraction(argc, argv);
    
    inf.readInt();
    int maxQ = inf.readInt();

    int t = inf.readInt();

    cout << t << endl << flush;
    tout << t << endl;

    for (int test_num = 0; test_num < t; ++test_num) {
        setTestCase(test_num + 1);

        string type = inf.readToken();

        auto interactor = ConstructInteractor(type);
        tout << interactor->GetK() << " ";
        int q_count = Simulate(std::move(interactor), maxQ);

        tout << q_count << endl;
    }

    return 0;
}
