#include "testlib.h"

#include "bits/stdc++.h"

#define rep(i, n) for(int i = 0; i < (n); ++i)
#define all(a) (a).begin(), (a).end()

using namespace std;

using ll = long long;

const int MIN_N = 1;
const int MAX_N = 800;

const int MIN_M = 1;
const int MAX_M = 3000;

const int MIN_P = 0;
const int MAX_P = 1'000'000'000;

const int MIN_W = 1;
const int MAX_W = 1'000'000'000;

const int MIN_S = 1;
const int MAX_S = 1'000'000'000;

const int maxng[7] = {MAX_N, MAX_N, MAX_N, 10, 100, 100, MAX_N};
const int maxsg[7] = {MAX_S, MAX_S, MAX_S, MAX_S, 100, MAX_S, MAX_S};
const int maxwg[7] = {MAX_W, 1, MAX_W, MAX_W, MAX_W, MAX_W, MAX_W};

int group;
int maxs, maxw;
int mins, minw;

vector<int> w;

struct Edge {
    int a, b, s;
};

vector<Edge> edges;
int n, m, p;

void GenWeights() {
    w.resize(n + 1);
    bool isSorted = opt<int>("wsort", 0);
    int max_change = opt<int>("max_change", maxw);

    int was_max = minw;
    for (int i = 1; i <= n; ++i) {
        w[i] = rnd.next(minw, min(maxw, was_max + max_change));
        was_max = max(was_max, w[i]);
    }

    if (isSorted) sort(w.begin() + 1, w.end());
}

void RandGen() {
    GenWeights();
    rep(_, m) {
        Edge e;
        e.a = rnd.next(1, n);
        do {
            e.b = rnd.next(1, n);
        } while (e.b == e.a);
        e.s = rnd.next(mins, maxs);
        edges.push_back(e);
    }
}

void GenArray() {
    m = n - 1;
    GenWeights();
    for (int i = 1; i < n; ++i) {
        Edge e;
        e.a = i;
        e.b = i + 1;
        e.s = rnd.next(mins, maxs);
        edges.push_back(e);
    }
}

void GenDAG() {
    GenWeights();
    rep(_, m) {
        Edge e;
        e.a = rnd.next(1, n);
        do {
            e.b = rnd.next(1, n);
        } while (e.b == e.a);
        if (e.a > e.b) swap(e.a, e.b);
        e.s = rnd.next(mins, maxs);
        edges.push_back(e);
    }
}

void GenChain() {
    GenWeights();
    vector<int> w2(w.begin() + 1, w.end());
    sort(all(w2));
    int v = 3;
    int k = 0;
    w[1] = w2[k++];
    while (v <= n) {
        w[v] = w2[k++];
        Edge e;
        e.a = v - 2;
        e.b = v;
        e.s = rnd.next(mins, maxs);
        edges.push_back(e);
        v += 2;
    }
    if (v != n) {
        w[n] = w2[k++];
        Edge e;
        e.a = n - 1;
        e.b = n;
        e.s = rnd.next(mins, maxs);
        edges.push_back(e);
    }
    for (int u = 2; u < n; u += 2) {
        for (int t = u - 1; t <= u; ++t) {
            Edge e;
            e.a = t;
            e.b = t + 1;
            e.s = rnd.next(mins, maxs);
            edges.push_back(e);
        }
        w[u] = w2[k++];
    }
    m = edges.size();
}

void GenDoubleChain() {
    GenWeights();
    vector<int> w2(w.begin() + 1, w.end());
    sort(all(w2));
    int v = 1;
    int k = 0;
    w[1] = w2[k++];
    while (v + 3 <= n) {
        v += 3;
        w[v] = w2[k++];
    }
    int antid = opt<int>("antid", 0);
    for (int t = 4; t <= v; t += 3) {
        for (int u = t - 2; u <= t - 1; ++u) {
            w[u] = w2[k++];
            int x;
            if (antid && (v - t) / 3 <= 30) {
                if (u == t - 2) x = 0;
                else x = min(maxs - mins, 1 << ((v - t) / 3));
                x += mins;
            } else {
                x = rnd.next(mins, maxs);
            }
            edges.push_back({t - 3, u, x});
            edges.push_back({u, t, x});
        }
    }
    while (v < n) {
        v++;
        w[v] = w2[k++];
        Edge e;
        e.a = v - 1;
        e.b = v;
        e.s = rnd.next(mins, maxs);
        edges.push_back(e);
    }
    m = edges.size();
}

void GenTree() {
    GenWeights();

    vector<int> w2(w.begin() + 1, w.end());
    sort(all(w2));
    int k = 0;

    int cnt_prev = opt<int>("cntp", n);
    vector<int> pr(n + 1);
    for (int i = 2; i <= n; ++i) {
        pr[i] = rnd.next(max(1, i - cnt_prev), i - 1);
    }

    vector<pair<int, int>> d(n + 1, {-1, -1});
    for (int v = n; v; v = pr[v]) d[v].first = 0;
    d[1].second = 0;
    for (int i = 2; i <= n; ++i) {
        d[i].second = d[pr[i]].second + 1;
        if (d[i].first == -1) d[i].first = d[pr[i]].first + 1;
    }
    vector<int> ord(n);
    iota(all(ord), 1);
    sort(all(ord), [&](const int &i, const int &j) {
        return d[i] < d[j];
    });
    for (auto &i: ord) w[i] = w2[k++];

    vector<int> wedges(2 * (n - 1));
    int avgs = (maxs - mins) / 2;
    for (auto &x: wedges) x = rnd.next(mins, mins + avgs);
    sort(all(wedges));
    reverse(all(wedges));
    int t = 0;

    for (auto &i: ord) {
        if (i == 1) continue;
        if (d[i].first == 0) {
            edges.push_back({pr[i], i, wedges[t++] + avgs});
            edges.push_back({i, pr[i], wedges[t++] + avgs});
        } else {
            edges.push_back({pr[i], i, wedges[t++]});
            edges.push_back({i, pr[i], wedges[t++]});
        }
    }

    m = (int) edges.size();
}

void Reorder(const vector<int> &perm) {
    vector<int> w2(n + 1);
    for (int v = 1; v <= n; ++v) w2[v] = w[perm[v]];
    w = w2;
    for (auto &e: edges) {
        assert(e.a <= n && e.b <= n && e.a >= 1 && e.b >= 1);
        e.a = perm[e.a];
        e.b = perm[e.b];
    }
}

void Shuffle() {
    vector<int> vord(n + 1);
    iota(all(vord), 0);
    shuffle(vord.begin() + 2, vord.begin() + n);
    Reorder(vord);
    shuffle(all(edges));
}

void Reverse() {
    vector<int> vord(n + 1);
    iota(all(vord), 0);
    reverse(vord.begin() + 2, vord.begin() + n);
    Reorder(vord);
}


int main(int argc, char *argv[]) {
    registerGen(argc, argv, 1);

    string mode = opt<string>("mode", "rand");
    group = opt<int>("group", 0);
    n = opt<int>("n", maxng[group]);
    m = opt<int>("m", MAX_M);
    p = opt<int>("p", MIN_P);
    maxw = opt<int>("wmax", maxwg[group]);
    maxs = opt<int>("smax", maxsg[group]);
    mins = opt<int>("smin", MIN_S);
    minw = opt<int>("wmin", MIN_W);

    if (mode == "rand") {
        RandGen();
    } else if (mode == "array") {
        GenArray();
    } else if (mode == "dag") {
        GenDAG();
    } else if (mode == "chain") {
        GenChain();
    } else if (mode == "double_chain") {
        GenDoubleChain();
    } else if (mode == "tree") {
        GenTree();
    }

    bool needReverse = opt<int>("rev", false);
    if (needReverse) Reverse();

    bool needShuffle = opt<int>("shuffle", group != 2 && !needReverse);
    if (needShuffle) Shuffle();

    cout << n << ' ' << m << ' ' << p << ' ' << group << '\n';

    for (int i = 1; i <= n; ++i) cout << w[i] << " \n"[i == n];

    for (int i = 0; i < m; ++i) {
        cout << edges[i].a << ' ' << edges[i].b << ' ' << edges[i].s << '\n';
    }
    return 0;
}
